from __future__ import absolute_import
__all__ = ['organ_segmentation']
#, 'qmisc', 'misc', 'experiments',
           #'support_structure_segmentation']
__version__ = "1.15.1"

import sys
import os.path as op
sys.path.insert(0, op.expanduser(r"~/projects/imtools/"))
sys.path.insert(0, op.expanduser(r"~/projects/seededitorqt/"))
from . import organ_segmentation
from .main import lisa_main
# import qmisc
# import misc
# import experiments
# import support_structure_segmentation
# from import Model, ImageGraphCut
# from seed_editor_qt import QTSeedEditor
# from dcmreaddata import DicomReader



"""Streaming pickle implementation for efficiently serializing and
de-serializing an iterable (e.g., list)

Created on 2010-06-19 by Philip Guo
  
http://code.google.com/p/streaming-pickle/

Modified by Brian Thorne 2013 to add base64 encoding to support
python3 bytearray and the like.
"""
import base64
from pickle import dumps, loads
import unittest
import tempfile

def s_dump(iterable_to_pickle, file_obj):
    """dump contents of an iterable iterable_to_pickle to file_obj, a file
    opened in write mode"""
    for elt in iterable_to_pickle:
        s_dump_elt(elt, file_obj)


def s_dump_elt(elt_to_pickle, file_obj):
    """dumps one element to file_obj, a file opened in write mode"""
    pickled_elt = dumps(elt_to_pickle)
    encoded = base64.b64encode(pickled_elt)
    file_obj.write(encoded)

    # record separator is a blank line
    # (since pickled_elt as base64 encoded cannot contain its own newlines)
    file_obj.write(b'\n\n')


def s_load(file_obj):
    """load contents from file_obj, returning a generator that yields one
    element at a time"""
    cur_elt = []
    for line in file_obj:

        if line == b'\n':
            encoded_elt = b''.join(cur_elt)
            try:
                pickled_elt = base64.b64decode(encoded_elt)
                elt = loads(pickled_elt)
            except EOFError:
                print("EOF found while unpickling data")
                print(pickled_elt)
                raise StopIteration
            cur_elt = []
            yield elt
        else:
            cur_elt.append(line)


class TestStreamingPickle(unittest.TestCase):
    def setUp(self):
        pass

    def testSimpleList(self):
        data = [1, 2, 3, 4, None, b'test', '\n', '\x00', 3, b'\n\n\n\n', 5, 7, 9, 11, "hello", bytearray([2, 4, 4])]
        with tempfile.TemporaryFile() as f:
            s_dump(data, f)
            # reset the temporary file
            f.seek(0)
            i = 0
            for i, element in enumerate(s_load(f)):
                self.assertEqual(data[i], element)
                # print(i, element)
            self.assertEqual(i, len(data)-1)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
import unittest
import os.path as op
from nose.plugins.attrib import attr
import io3d.datasets
from lisa import runner
import lisa.autolisa


class MyClass:
    def __init__(self):
        self.retval = None
        pass

    def return_string(self):
        self.retval = "string"
        return "string"

    def return_int(self):
        self.retval = 1
        return 1

    def return_third_param(self, first, second, third):
        self.retval = third


class RunnerTest(unittest.TestCase):

    def test_runner(self):
        mcls = MyClass()
        rnr = runner.Runner(mcls)
        rnr.extend(["return_string", "return_int"])
        rnr.run()

        self.assertEqual(mcls.retval, 1)


    def test_runner_by_function(self):
        mcls = MyClass()
        rnr = runner.Runner(mcls)
        rnr.extend([mcls.return_string])
        rnr.run()

        self.assertEqual(mcls.retval, "string")

    def test_runner_complex(self):

        mcls = MyClass()
        rnr = runner.Runner(mcls)
        rnr.extend([mcls.return_string, ["return_third_param",[1, 2, 3], {}]])
        rnr.run()

        self.assertEqual(mcls.retval, 3)



    def test_lisa_auto(self):
        """
        Interactivity is stored to file
        """
        dcmdir = io3d.datasets.join_path("medical", "orig", '3Dircadb1.*1', "PATIENT_DICOM", get_root=True)
        import glob
        pths = glob.glob(dcmdir)
        al = lisa.autolisa.AutoLisa()
        al.run_in_paths(dcmdir)
        output_paths = glob.glob(op.expanduser("~/lisa_data/"))

        self.assertGreater(len(output_paths), 0)






#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR% %USER% <%MAIL%>
#
# Distributed under terms of the %LICENSE% license.

"""
%HERE%
"""

import logging

logger = logging.getLogger(__name__)
import os.path
import sys
path_to_script = os.path.dirname(os.path.abspath(__file__))
pth = os.path.join(path_to_script, "../../seededitorqt/")
sys.path.insert(0, pth)
pth = os.path.join(path_to_script, "../../imtools/")
sys.path.insert(0, pth)

import argparse

def lisa_main():
    import argparse

    parser = argparse.ArgumentParser(
        # Turn off help, so we print all options in response to -h
        add_help=False
    )
    parser.add_argument(
        '-ni', '--no_interactivity', action='store_true',
        help='run in no interactivity mode, seeds must be defined')
    parser.add_argument(
        '--autolisa',
        action='store_true',
        help='run autolisa in dir',
        default=False
    )
    knownargs, unknownargs = parser.parse_known_args()
    # Read alternative config file. First is loaded default config. Then user
    # config in lisa_data directory. After that is readed config defined by
    # --configfile parameter

    if knownargs.no_interactivity or knownargs.autolisa:
        import organ_segmentation
        organ_segmentation.main()
    else:

        import PyQt4
        import PyQt4.QtGui
        import sys
        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        # Create and display the splash screen
        from . import splash_screen
        splash = splash_screen.splash_screen(app)
        from . import organ_segmentation
        organ_segmentation.main(app, splash)


if __name__ == "__main__":
    lisa_main()


#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
import unittest
import os.path as op
from nose.plugins.attrib import attr

path_to_script = op.dirname(op.abspath(__file__))

import sys

sys.path.insert(0, op.abspath(op.join(path_to_script, "../../io3d")))
sys.path.insert(0, op.abspath(op.join(path_to_script, "../../imma")))
# import sys
# import os.path

# imcut_path =  os.path.join(path_to_script, "../../imcut/")
# sys.path.insert(0, imcut_path)
import lisa.virtual_resection
import numpy as np


class ResectionTest(unittest.TestCase):

    # @unittest.skipIf(True, "Skipped test")
    # @attr("slow")
    @unittest.skip("demonstrating skipping")
    def test_nothing(self):
        self.fail("shouldn't happen")

    @attr('interactive')
    def test_planar_resection(self):
        """
        Select points for planar virtual resection and check if it works properly
        :return:
        """
        shape = [40, 41, 42]
        from lisa.dataset import generate_sample_data
        datap = generate_sample_data()
        import sys
        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        print("Select points for planar virtual resection")
        datap = lisa.virtual_resection.resection_planar(datap, interactivity=True, seeds=None)

    def test_planar_resection2(self):
        """
        Check planar virtual resection
        :return:
        """
        shape = [40, 41, 42]
        from lisa.dataset import generate_sample_data
        datap = generate_sample_data(shape=shape)
        seeds = np.zeros(shape)
        inds = np.asarray([[10, 5, 20, 22], [20, 8, 10, 12], [5, 10, 20, 19]])
        seeds[inds[0], inds[1], inds[2]] = 1

        datap = lisa.virtual_resection.resection_planar(datap, interactivity=False, seeds=seeds)

        sonda1 = datap['segmentation'][11, 5, 4]
        sonda2 = datap['segmentation'][22, 22, 14]

        liver_labels = [datap['slab']['liver'], datap['slab']['resected_liver']]
        self.assertIn(sonda1, liver_labels)
        self.assertIn(sonda2, liver_labels)
        self.assertNotEqual(sonda1, sonda2)

    def test_portal_vein_new_resection(self):
        """
        Check planar virtual resection
        :return:
        """
        shape = [40, 41, 42]
        from lisa.dataset import generate_sample_data
        datap = generate_sample_data(shape=shape)
        seeds = np.zeros(shape)
        # points in portal vein where cut will be performed
        inds = np.asarray([[15, 15], [15, 15], [8, 9]])
        seeds[inds[0], inds[1], inds[2]] = 1

        datap = lisa.virtual_resection.resection_portal_vein_new(
            datap, interactivity=False, seeds=seeds)

        sonda1 = datap['segmentation'][11, 5, 4]
        sonda2 = datap['segmentation'][22, 22, 14]

        liver_labels = [datap['slab']['liver'], datap['slab']['resected_liver']]
        # import sed3
        # ed = sed3.sed3(datap['segmentation'])
        # ed.show()
        self.assertIn(sonda1, liver_labels)
        self.assertIn(sonda2, liver_labels)
        self.assertNotEqual(sonda1, sonda2)

    def test_portal_vein_resection(self):
        """
        Check planar virtual resection
        :return:
        """
        shape = [40, 41, 42]
        from lisa.dataset import generate_sample_data
        datap = generate_sample_data(shape=shape)
        seeds = np.zeros(shape)
        # points in portal vein where cut will be performed
        inds = np.asarray([[15, 15], [15, 15], [8, 9]])
        seeds[inds[0], inds[1], inds[2]] = 1

        # import sed3
        # ed = sed3.sed3(datap["data3d"], seeds=seeds, contour=datap['segmentation'])
        # ed.show()
        datap = lisa.virtual_resection.resection_old(datap, interactivity=False, seeds=seeds)

        sonda1 = datap['segmentation'][11, 5, 4]
        sonda2 = datap['segmentation'][22, 22, 14]

        liver_labels = [datap['slab']['liver'], datap['slab']['resected_liver']]
        self.assertIn(sonda1, liver_labels)
        self.assertIn(sonda2, liver_labels)
        self.assertNotEqual(sonda1, sonda2)
        # import sed3
        # ed = sed3.sed3(datap['segmentation'])
        # ed.show()

    def test_branch_labels_just_in_module(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        un0 = np.unique(oseg.segmentation)
        keys0 = list(oseg.slab.keys())
        bl = lisa.virtual_resection.label_volumetric_vessel_tree(oseg, "porta")
        un1 = np.unique(oseg.segmentation)
        keys1 = list(oseg.slab.keys())

        # import sed3
        # ed = sed3.sed3(bl, contour=datap["segmentation"])
        # ed.show()

        self.assertGreater(len(un1), len(un0))
        self.assertGreater(len(keys1), len(keys0))

    def test_branch_labels_from_oseg(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        un0 = np.unique(oseg.segmentation)
        keys0 = list(oseg.slab.keys())
        oseg.label_volumetric_vessel_tree("porta")
        un1 = np.unique(oseg.segmentation)
        keys1 = list(oseg.slab.keys())
        # import sed3
        # ed = sed3.sed3(oseg.segmentation)
        # ed.show()

        self.assertGreater(len(un1), len(un0))
        self.assertGreater(len(keys1), len(keys0))

    def test_branch_labels_with_gui_just_in_module(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        labeled_branches = lisa.virtual_resection.label_volumetric_vessel_tree(oseg, "porta")
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        slab = datap["slab"]
        organ_label = "liver"

        seeds = np.zeros_like(data3d, dtype=np.int)
        seeds[40, 125, 166] = 1
        seeds[40, 143, 130] = 2
        seeds[40, 125, 115] = 3

        seglabel1 = labeled_branches[seeds == 1][0]
        seglabel2 = labeled_branches[seeds == 2][0]
        seglabel3 = labeled_branches[seeds == 3][0]
        import imma.measure
        import imma.image_manipulation
        import imma.image_manipulation as ima

        # import sed3
        # ed = sed3.sed3(labeled_branches)  # , contour=datap["segmentation"])
        # ed.show()
        organseg = ima.select_labels(segmentation, organ_label, slab)

        organ_split, connected = lisa.virtual_resection.split_tissue_on_labeled_tree(
            labeled_branches, seglabel1, [seglabel2, seglabel3], organseg
        )

        # import sed3
        # # ed = sed3.sed3(labeled_branches, contour=organ_split)
        # ed = sed3.sed3(organ_split)
        # ed.show()

        self.assertTrue(np.array_equal(np.unique(organ_split), [0, 1, 2]))

        self.assertGreater(np.sum(organ_split == 0), 1000, "At least some background expected")
        self.assertGreater(np.sum(organ_split == 1), 1000, "At least some object expected")
        self.assertGreater(np.sum(organ_split == 2), 1000, "At least some object expected")

    def test_split_organ_segmentation(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        slab = datap["slab"]
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        oseg.label_volumetric_vessel_tree("porta")
        labeled_branches = oseg.segmentation
        seeds = np.zeros_like(oseg.data3d, dtype=np.int)
        seeds[40, 125, 166] = 1
        seeds[40, 143, 130] = 2
        seeds[40, 125, 115] = 3

        seglabel1 = labeled_branches[seeds == 1][0]
        seglabel2 = labeled_branches[seeds == 2][0]
        seglabel3 = labeled_branches[seeds == 3][0]
        split_labels_out, connected = oseg.split_tissue_with_labeled_volumetric_vessel_tree("liver",
                                                                                            seglabel1, [seglabel2, seglabel3],
                                                                                            split_labels=["split1", "split2"]
                                                                                            )
        # labeled_branches = lisa.virtual_resection.branch_labels(oseg, "porta")
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        organ_label = "liver"



        # import sed3
        # # ed = sed3.sed3(labeled_branches, contour=organ_split)
        # ed = sed3.sed3(organ_split)
        # ed.show()

        self.assertGreater(np.sum(oseg.select_label("split1")), 1000)
        self.assertGreater(np.sum(oseg.select_label("split2")), 1000)

    def generate_trifurcation(self):
        import io3d.datasets
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        slab = datap["slab"]
        # third missed branch
        third_branch_location = [slice(40, 44), slice(90, 120), slice(128, 133)]
        datap["segmentation"][third_branch_location] = slab['porta']
        datap["data3d"][third_branch_location] += 206
        return datap

    def test_split_organ_segmentation_missed_branch(self):
        import lisa.organ_segmentation
        datap = self.generate_trifurcation()
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        # import sed3
        # ed = sed3.sed3(oseg.data3d, contour=oseg.segmentation)
        # ed.show()
        oseg.label_volumetric_vessel_tree("porta")
        labeled_branches = oseg.segmentation
        seeds = np.zeros_like(oseg.data3d, dtype=np.int)
        seeds[40, 125, 166] = 1
        seeds[40, 143, 130] = 2
        seeds[40, 125, 115] = 3

        seglabel1 = labeled_branches[seeds == 1][0]
        seglabel2 = labeled_branches[seeds == 2][0]
        seglabel3 = labeled_branches[seeds == 3][0]
        split_labels_out, connected = oseg.split_tissue_with_labeled_volumetric_vessel_tree("liver",
                                                                                            seglabel1, [seglabel2, seglabel3],
                                                                                            split_labels=["split1", "split2"],
                                                                                            on_missed_branch="split"
                                                                                            )

        # import sed3
        # ed = sed3.sed3(oseg.data3d, contour=oseg.select_label("split1"))
        # ed.show()

        self.assertGreater(np.sum(oseg.select_label("split1")), 1000)
        self.assertGreater(np.sum(oseg.select_label("split2")), 1000)

    def test_split_organ_segmentation_missed_branch_neighboor_to_branches_organ_label(self):
        import lisa.organ_segmentation
        datap = self.generate_trifurcation()
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        # import sed3
        # ed = sed3.sed3(oseg.data3d, contour=oseg.segmentation)
        # ed.show()
        oseg.label_volumetric_vessel_tree("porta")
        labeled_branches = oseg.segmentation
        # import sed3
        # ed = sed3.sed3(oseg.segmentation, contour=oseg.segmentation)
        # ed.show()
        seeds = np.zeros_like(oseg.data3d, dtype=np.int)
        seeds[40, 125, 166] = 1
        seeds[40, 143, 130] = 2
        seeds[42, 100, 130] = 3

        seglabel1 = labeled_branches[seeds == 1][0]
        seglabel2 = labeled_branches[seeds == 2][0]
        seglabel3 = labeled_branches[seeds == 3][0]
        split_labels_out, connected = oseg.split_tissue_with_labeled_volumetric_vessel_tree(
            "liver", seglabel1, [seglabel2, seglabel3],
            split_labels=["split1", "split2"],
            on_missed_branch="orig"
        )

        # import sed3
        # # ed = sed3.sed3(oseg.data3d, contour=oseg.select_label("split1"))
        # ed = sed3.sed3(oseg.segmentation, contour=oseg.select_label("split1"))
        # ed.show()

        # self.assertEqual(len(split_labels_out), 3)
        self.assertGreater(np.sum(oseg.select_label("split1")), 1000)
        self.assertGreater(np.sum(oseg.select_label("split2")), 1000)
        self.assertGreater(np.sum(oseg.select_label("liver")), 1000)

    def test_split_organ_segmentation_missed_branch_neighboor_to_trunk_organ_label(self):
        import lisa.organ_segmentation
        datap = self.generate_trifurcation()
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        # import sed3
        # ed = sed3.sed3(oseg.data3d, contour=oseg.segmentation)
        # ed.show()
        oseg.label_volumetric_vessel_tree("porta")
        labeled_branches = oseg.segmentation
        # import sed3
        # ed = sed3.sed3(oseg.segmentation, contour=oseg.segmentation)
        # ed.show()
        seeds = np.zeros_like(oseg.data3d, dtype=np.int)
        seeds[40, 125, 166] = 1
        seeds[40, 143, 130] = 2
        seeds[40, 125, 115] = 3

        seglabel1 = labeled_branches[seeds == 1][0]
        seglabel2 = labeled_branches[seeds == 2][0]
        seglabel3 = labeled_branches[seeds == 3][0]
        split_labels_out, connected = oseg.split_tissue_with_labeled_volumetric_vessel_tree(
            "liver", seglabel1, [seglabel2, seglabel3],
            split_labels=["split1", "split2"],
            on_missed_branch="orig"
        )

        # import sed3
        # ed = sed3.sed3(oseg.data3d, contour=oseg.select_label("split1"))
        # ed.show()

        # self.assertEqual(len(split_labels_out), 3)
        self.assertGreater(np.sum(oseg.select_label("split1")), 1000)
        self.assertGreater(np.sum(oseg.select_label("split2")), 1000)
        self.assertGreater(np.sum(oseg.select_label("liver")), 1000)

    @unittest.skip("Waiting for implementation of recursive tissue segmentation")
    def test_split_organ_segmentation_recursive(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        slab = datap["slab"]
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        oseg.label_volumetric_vessel_tree("porta")
        labeled_branches = oseg.segmentation
        seeds = np.zeros_like(oseg.data3d, dtype=np.int)
        seeds[40, 122:126, 165:168] = 1
        seeds[40, 144:148, 131:133] = 2
        seeds[40, 122:126, 114:117] = 2
        seeds[40, 90:95, 103:106] = 3
        seeds[40, 122:126, 84:88] = 3


        # import sed3
        # ed = sed3.sed3(datap["data3d"], contour=datap["segmentation"], seeds=seeds)
        # ed.show()

        split_labels_out, connected = oseg.split_tissue_recusively_with_labeled_volumetric_vessel_tree("liver", seeds)
        # labeled_branches = lisa.virtual_resection.branch_labels(oseg, "porta")
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        organ_label = "liver"

        import sed3
        # ed = sed3.sed3(labeled_branches, contour=organ_split)
        ed = sed3.sed3(split_labels_out)
        ed.show()

        self.assertGreater(np.sum(oseg.select_label("split1")), 1000)
        self.assertGreater(np.sum(oseg.select_label("split2")), 1000)


if __name__ == "__main__":
    # logging.basicConfig(stream=sys.stderr)
    logger.setLevel(logging.DEBUG)
    unittest.main()



# ! /usr/bin/python
# -*- coding: utf-8 -*-
import logging
logger = logging.getLogger(__name__)

import numpy as np


class GMMClassifier():

    def __init__(self, each_class_params=None, **same_params):
        """
        same_params: classifier params for each class are same
        each_class_params: is list of dictionary of params for each
            class classifier. For example:
            [{'covariance_type': 'full'}, {'n_components': 2}])
        """
        self.same_params = same_params
        self.each_class_params = each_class_params

        self.models = []

    def fit(self, X_train, y_train):
        X_train = np.asarray(X_train)
        y_train = np.asarray(y_train)
        # from sklearn.mixture import GMM as GaussianMixture
        from sklearn.mixture import GaussianMixture

        unlabels = range(0, np.max(y_train) + 1)

        for lab in unlabels:
            if self.each_class_params is not None:
                # print 'eacl'
                # print self.each_class_params[lab]
                model = GaussianMixture(**self.each_class_params[lab])
                # print 'po gmm ', model
            elif len(self.same_params) > 0:
                model = GaussianMixture(**self.same_params)
                # print 'ewe ', model
            else:
                model = GaussianMixture()
            X_train_lab = X_train[y_train == lab]
            # logger.debug('xtr lab shape ' + str(X_train_lab))
            model.fit(X_train_lab)

            self.models.insert(lab, model)

    def __str__(self):
        if self.each_class_params is not None:
            return "GMMClassificator(" + str(self.each_class_params) + ')'
        else:
            return "GMMClassificator(" + str(self.same_params) + ')'

    def predict(self, X_test):
        X_test = np.asarray(X_test)

        logger.debug(str(X_test.shape))
        logger.debug(str(X_test))
        scores = np.zeros([X_test.shape[0], len(self.models)])

        for lab in range(0, len(self.models)):

            logger.debug('means shape' + str(self.models[lab].means_.shape))
            sc = self.models[lab].score_samples(X_test)
            scores[:, lab] = sc

        pred = np.argmax(scores, 1)

        return pred



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Simple VTK Viewer.

Example:

$ viewer.py -f head.vtk
"""
from optparse import OptionParser
import sys

from PyQt4.QtGui import QApplication, QDialog, QGridLayout, QPushButton
import vtk
from vtk.qt4.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor


class QVTKViewer(QDialog):
    """
    Simple VTK Viewer.
    """

    def initUI(self):

        grid = QGridLayout()
        self.vtkWidget = QVTKRenderWindowInteractor(self)
        grid.addWidget(self.vtkWidget, 0, 0, 1, 1)

        btn_close = QPushButton("close", self)
        btn_close.clicked.connect(self.close)
        grid.addWidget(btn_close, 1, 0, 1, 1)

        self.setLayout(grid)
        self.setWindowTitle('VTK Viewer')
        self.show()

    def __init__(self, vtk_filename=None, vtk_data=None):
        """
        Initiate Viwer

        Parameters
        ----------
        vtk_filename : str
            Input VTK filename
        """

        QDialog.__init__(self)
        self.initUI()

        ren = vtk.vtkRenderer()
        self.vtkWidget.GetRenderWindow().AddRenderer(ren)
        iren = self.vtkWidget.GetRenderWindow().GetInteractor()

        if vtk_filename is not None:
            # VTK file
            reader = vtk.vtkUnstructuredGridReader()
            reader.SetFileName(vtk_filename)
            reader.Update()
            vtkdata = reader.GetOutput()

        if vtk_data is not None:
            vtkdata = vtk_data

        # VTK surface
        surface = vtk.vtkDataSetSurfaceFilter()
        surface.SetInputData(vtkdata)
        # surface.SetInput(vtkdata)
        surface.Update()

        mapper = vtk.vtkDataSetMapper()
        mapper.SetInputData(surface.GetOutput())

        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        # actor.GetProperty().EdgeVisibilityOff()
        actor.GetProperty().SetEdgeColor(1,0.0,1)
        actor.GetProperty().SetDiffuseColor(1,0.0,1.0)
        actor.GetProperty().SetAmbientColor(1,0.0,1)
        actor.GetProperty().SetLineWidth(0.1)
        # import pdb; pdb.set_trace()
        # actor.GetProperty().SetColor(1, 0, 1)
        actor.GetProperty().SetOpacity(0.3)

        ren.AddActor(actor)

        # annot. cube
        axesActor = vtk.vtkAnnotatedCubeActor()
        axesActor.SetXPlusFaceText('R')
        axesActor.SetXMinusFaceText('L')
        axesActor.SetYMinusFaceText('H')
        axesActor.SetYPlusFaceText('F')
        axesActor.SetZMinusFaceText('A')
        axesActor.SetZPlusFaceText('P')
        axesActor.GetTextEdgesProperty().SetColor(1, 0, 0)
        axesActor.GetCubeProperty().SetColor(0, 0, 1)
        self.axes = vtk.vtkOrientationMarkerWidget()
        self.axes.SetOrientationMarker(axesActor)
        self.axes.SetInteractor(iren)
        self.axes.EnabledOn()
        self.axes.InteractiveOn()

        ren.SetBackground(0.5, 0.5, 0.5)
        ren.ResetCamera()
        iren.Initialize()


usage = '%prog [options]\n' + __doc__.rstrip()
help = {
    'in_file': 'input VTK file with unstructured mesh',
}


def main():
    parser = OptionParser(description='Simple VTK Viewer')
    parser.add_option('-f', '--filename', action='store',
                      dest='in_filename', default=None,
                      help=help['in_file'])
    (options, args) = parser.parse_args()

    if options.in_filename is None:
        raise IOError('No VTK data!')

    app = QApplication(sys.argv)
    viewer = QVTKViewer(options.in_filename)
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@hp-mjirik>
#
# Distributed under terms of the MIT license.

"""

"""
import logging
logger = logging.getLogger(__name__)
from nose.plugins.attrib import attr
import numpy as np
import scipy
import lisa.data_manipulation as dama
import unittest


class DataManipulationTest(unittest.TestCase):

    @attr('actual')
    def test_unbiased_brick(self):
        """
        Test unbiased brick. There sould be correct number of object in
        sum of subbrick.
        """
# import np.random
        shp = [40, 42, 44]
        data = np.zeros(shp, dtype=np.uint8)
        data[14:18, 22, 17] = 1
        data[12:16, 15, 15] = 1
        data[13:18, 18, 13] = 1
        # data[26:29, 21, 18] = 1
        # data[21, 23:26, 25] = 1
        # data[21, 17:21, 24] = 1
        # data[16, 26:29, 21] = 1
        # data[24, 28, 13:23] = 1
        # data[19, 28, 12:16] = 1
        # data[19, 15, 22:26] = 1


        # import sed3
        # se = sed3.sed3(data)
        # se.show()

# pokud to pocita dobr
        sh = [None] * 8
        sh[0] = [[10,20],[10,20],[10,20]]
        sh[1] = [[10,20],[10,20],[20,30]]
        sh[2] = [[10,20],[20,30],[10,20]]
        sh[3] = [[10,20],[20,30],[20,30]]
        sh[4] = [[20,30],[10,20],[10,20]]
        sh[5] = [[20,30],[10,20],[20,30]]
        sh[6] = [[20,30],[20,30],[10,20]]
        sh[7] = [[20,30],[20,30],[20,30]]

        suma = 0

        for shi in sh:

            outputdata = dama.unbiased_brick_filter(data, shi)
            imlab, num_features = scipy.ndimage.measurements.label(outputdata)
            logger.debug(num_features)
            suma += num_features

        logger.debug("for is over")

        logger.debug("suma " + str(suma))
        # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT


        

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

# import sys
import os

import logging
logger = logging.getLogger(__name__)

import sys
import os.path

import numpy as np
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "./extern/sPickle"))
# import numpy as np

from io3d.misc import suggest_filename
from io3d.misc import obj_from_file
from io3d.misc import obj_to_file

# def suggest_filename(file_path, exists=None):
#     """
#     Try if exist path and append number to its end.
#     For debug you can set as input if file exists or not.
#     """
#     import os.path
#     import re
#     if not isinstance(exists, bool):
#         exists = os.path.exists(file_path)
#
#     if exists:
#         file_path, file_extension = os.path.splitext(file_path)
#         # print file_path
#         m = re.search(r"\d+$", file_path)
#         if m is None:
#             # cislo = 2
#             new_cislo_str = "2"
#         else:
#             cislostr = (m.group())
#             cislo = int(cislostr) + 1
#             file_path = file_path[:-len(cislostr)]
#             new_cislo_str = str(cislo)
#
#         file_path = file_path + new_cislo_str + file_extension  # .zfill(2)
#         # trorcha rekurze
#         file_path = suggest_filename(file_path)
#
#     return file_path
#
#
# def obj_from_file(filename='annotation.yaml', filetype='yaml'):
#     ''' Read object from file '''
#
#     if filetype == 'auto':
#         _, ext = os.path.splitext(filename)
#         filetype = ext[1:]
#
#     if filetype == 'yaml':
#         import yaml
#         f = open(filename, 'rb')
#         obj = yaml.load(f)
#         f.close()
#     elif filetype in ('pickle', 'pkl', 'pklz', 'picklezip'):
#         fcontent = read_pkl_and_pklz(filename)
#         # import pickle
#         import cPickle as pickle
#         # import sPickle as pickle
#         obj = pickle.loads(fcontent)
#     else:
#         logger.error('Unknown filetype ' + filetype)
#     return obj
#
#
# def read_pkl_and_pklz(filename):
#     """
#     Try read zipped or not zipped pickle file
#     """
#     fcontent = None
#     try:
#         import gzip
#         f = gzip.open(filename, 'rb')
#         fcontent = f.read()
#         f.close()
#     except IOError as e:
#         # if the problem is in not gzip file
#         logger.warning("Input gzip exception: " + str(e))
#         f = open(filename, 'rb')
#         fcontent = f.read()
#         f.close()
#     except Exception as e:
#         # other problem
#         import traceback
#         logger.error("Input gzip exception: " + str(e))
#         logger.error(traceback.format_exc())
#
#     return fcontent
#
# from io3d.misc import obj_to_file
# def obj_to_file(obj, filename='annotation.yaml', filetype='yaml'):
#     '''Writes annotation in file.
#
#     Filetypes:
#         yaml
#         pkl, pickle
#         pklz, picklezip
#     '''
#     # import json
#     # with open(filename, mode='w') as f:
#     #    json.dump(annotation,f)
#
#     # write to yaml
#     d = os.path.dirname(os.path.abspath(filename))
#     if not os.path.exists(d):
#         os.makedirs(d)
#
#     if filetype == 'auto':
#         _, ext = os.path.splitext(filename)
#         filetype = ext[1:]
#
#     if filetype == 'yaml':
#         f = open(filename, 'wb')
#         import yaml
#         yaml.dump(obj, f)
#         f.close
#     elif filetype in ('pickle', 'pkl'):
#         f = open(filename, 'wb')
#         import cPickle as pickle
#         pickle.dump(obj, f)
#         f.close
#     elif filetype in ('streamingpicklezip', 'spklz'):
#         # this is not working :-(
#         import gzip
#         import sPickle as pickle
#         f = gzip.open(filename, 'wb', compresslevel=1)
#         # f = open(filename, 'wb')
#         pickle.s_dump(obj, f)
#         f.close
#     elif filetype in ('picklezip', 'pklz'):
#         import gzip
#         import cPickle as pickle
#         f = gzip.open(filename, 'wb', compresslevel=1)
#         # f = open(filename, 'wb')
#         pickle.dump(obj, f)
#         f.close
#     elif filetype in('mat'):
#
#         import scipy.io as sio
#         sio.savemat(filename, obj)
#     else:
#         logger.error('Unknown filetype ' + filetype)

from io3d.misc import resize_to_shape
# def resize_to_shape(data, shape, zoom=None, mode='nearest', order=0):
#     """
#     Function resize input data to specific shape.
#
#     :param data: input 3d array-like data
#     :param shape: shape of output data
#     :param zoom: zoom is used for back compatibility
#     :mode: default is 'nearest'
#     """
#     # @TODO remove old code in except part
#
#     try:
#         # rint 'pred vyjimkou'
#         # aise Exception ('test without skimage')
#         # rint 'za vyjimkou'
#         import skimage
#         import skimage.transform
# # Now we need reshape  seeds and segmentation to original size
#
#         segm_orig_scale = skimage.transform.resize(
#             data, shape, order=0,
#             preserve_range=True
#         )
#
#         segmentation = segm_orig_scale
#         logger.debug('resize to orig with skimage')
#     except:
#         import scipy
#         import scipy.ndimage
#         dtype = data.dtype
#         if zoom is None:
#             zoom = shape / np.asarray(data.shape).astype(np.double)
#
#         segm_orig_scale = scipy.ndimage.zoom(
#             data,
#             1.0 / zoom,
#             mode=mode,
#             order=order
#         ).astype(dtype)
#         logger.debug('resize to orig with scipy.ndimage')
#
# # @TODO odstranit hack pro oříznutí na stejnou velikost
# # v podstatě je to vyřešeno, ale nechalo by se to dělat elegantněji v zoom
# # tam je bohužel patrně bug
#         # rint 'd3d ', self.data3d.shape
#         # rint 's orig scale shape ', segm_orig_scale.shape
#         shp = [
#             np.min([segm_orig_scale.shape[0], shape[0]]),
#             np.min([segm_orig_scale.shape[1], shape[1]]),
#             np.min([segm_orig_scale.shape[2], shape[2]]),
#         ]
#         # elf.data3d = self.data3d[0:shp[0], 0:shp[1], 0:shp[2]]
#         # mport ipdb; ipdb.set_trace() # BREAKPOINT
#
#         segmentation = np.zeros(shape, dtype=dtype)
#         segmentation[
#             0:shp[0],
#             0:shp[1],
#             0:shp[2]] = segm_orig_scale[0:shp[0], 0:shp[1], 0:shp[2]]
#
#         del segm_orig_scale
#     return segmentation



# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
List of functions to run
"""
import logging
logger = logging.getLogger(__name__)

import glob
import os.path as op

from . import organ_segmentation


class AutoLisa:
    def __init__(self):
        self.config = None
        pass

    def run_one(self, input_data_path, output_datapath=None):
        import os.path as op
        import io3d.datasets

        # input_data_path = io3d.datasets.join_path("jatra_5mm/")


        oseg = organ_segmentation.OrganSegmentation(
            # input_annotation_file=input_annotation_file,
            datapath=input_data_path,
            get_series_number_callback="guess for liver",
            # output_annotation_file="output.json",
            autocrop=False,
            run_list=["get_body_navigation_structures_precise", "save_outputs"],
            output_datapath=output_datapath

        )
        oseg.make_run()
        # oseg.sliver_compare_with_other_volume_from_file("file.pklz")

    def run_in_paths(self, path):
        if type(path) in (list, tuple):
            files = path
        else:
            files = glob.glob(path)
        for pth in files:
            self.run_one(pth)









#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import os.path
import sys

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
# from ..extern.sed3 import sed3
# import featurevector

import logging
logger = logging.getLogger(__name__)

import numpy as np
import scipy.ndimage
# import vtk
import argparse
# @TODO remove logger debug message from the header
logger.debug("before morphology import")
from skimage import morphology

# from PyQt4 import QtCore, QtGui
# from PyQt4.QtGui import *
# from PyQt4.QtCore import Qt
# from PyQt4.QtGui import QApplication
# from PyQt4.QtGui import QApplication, QMainWindow, QWidget,\
#     QGridLayout, QLabel, QPushButton, QFrame, QFileDialog,\
#     QFont, QInputDialog, QComboBox, QRadioButton, QButtonGroup

# ----------------- my scripts --------
from . import misc
import sed3
# import show3
from . import qmisc
from . import data_manipulation
import imma.image_manipulation as ima


def resection(data, name=None, method='PV',
              interactivity=True, seeds=None, **kwargs):

    """
    Main resection function.

    :param data: dictionaru with data3d, segmentation and slab key.
    :param method: "PV", "planar"
    :param interactivity: True or False, use seeds if interactivity is False
    :param seeds: used as initial interactivity state
    :param kwargs: other parameters for resection algorithm
    :return:
    """
    if method is 'PV':
        return resection_old(data, interactivity=interactivity, seeds=seeds)
    elif method is 'planar':
        return resection_planar(data, interactivity=interactivity, seeds=seeds)
    elif method is "PV_new":
        return resection_portal_vein_new(data, interactivity=interactivity, seeds=seeds, organ_label=data["slab"]["liver"], vein_label=data["slab"]["porta"])
        # return resection_portal_vein_new(data, interactivity=interactivity, seeds=seeds, **kwargs)
    else:
        return resection_with_3d_visualization(data, **kwargs)


def Rez_podle_roviny(plane, data, voxel):

    a = plane.GetNormal()[0] * voxel[0]
    b = plane.GetNormal()[1] * voxel[1]
    c = plane.GetNormal()[2] * voxel[2]
    xx = plane.GetOrigin()[0] / voxel[0]
    yy = plane.GetOrigin()[1] / voxel[1]
    zz = plane.GetOrigin()[2] / voxel[2]
    d = -(a * xx) - (b * yy) - (c * zz)
    mensi = 0
    vetsi = 0
    mensi_objekt = 0
    vetsi_objekt = 0
    print('x: ', a, ' y: ', b, ' z: ', c)
    print('Pocitani rezu...')
    prava_strana = np.ones((data.shape[0], data.shape[1], data.shape[2]))
    leva_strana = np.ones((data.shape[0], data.shape[1], data.shape[2]))
    dimension = data.shape
    for x in range(dimension[0]):
        for y in range(dimension[1]):
            for z in range(dimension[2]):
                rovnice = a * x + b * y + c * z + d
                if((rovnice) <= 0):
                    mensi = mensi + 1
                    if(data[x][y][z] == 1):
                        mensi_objekt = mensi_objekt + 1
                    leva_strana[x][y][z] = 0
                else:
                    vetsi = vetsi + 1
                    if(data[x][y][z] == 1):
                        vetsi_objekt = vetsi_objekt + 1
                    prava_strana[x][y][z] = 0
    leva_strana = leva_strana * data
    objekt = mensi_objekt + vetsi_objekt
    odstraneni_procenta = ((100 * mensi_objekt) / objekt)
    print(leva_strana)

    return leva_strana, odstraneni_procenta


# ----------------------------------------------------------
def cut_editor_old(data, label=None):
    logger.debug("editor input label: " + str(label))

    if label is None:
        contour=data['segmentation']
    else:
        if type(label) == str:
            label = data['slab'][label]
        contour=(data['segmentation'] == label).astype(np.int8)
    pyed = sed3.sed3qt(data['data3d'], contour=contour)
    pyed.exec_()

    return pyed.seeds


def split_vessel(datap, seeds, vessel_volume_threshold=0.95, dilatation_iterations=1, input_label="porta",
                 output_label1 = 1, output_label2 = 2, input_seeds_cut_label=1,
                 input_seeds_separate_label=3,
                 input_seeds_label2=None,
                 method="reach volume",
                 ):
    """

    :param datap: data plus format with data3d, segmentation, slab ...
    :param seeds: 3d ndarray same size as data3d, label 1 is place where should be vessel cuted. Label 2 points to
    the vessel with output label 1 after the segmentation
    :param vessel_volume_threshold: this parameter defines the iteration stop rule if method "reach volume is selected
    :param dilatation_iterations:
    :param input_label: which vessel should be splited
    :param output_label1: output label for vessel part marked with right button (if it is used)
    :param output_label2: ouput label for not-marked vessel part
    :param method: "separate labels" or "reach volume". The first method needs 3 input seeds and it is more stable.
    :param input_seeds_separate_label: after the segmentation the object containing this label in seeds would be labeled with
    output_label1
    :param input_seeds_label2: This parameter is usedf the method is "separate labels". After the
    segmentation the object containing this label in seeds would be labeled with output_label1.
    :return:
    """
    split_obj0 = (seeds == input_seeds_cut_label).astype(np.int8)
    split_obj = split_obj0.copy()


    # numeric_label = imma.get_nlabel(datap["slab"], input_label)
    if method == "separate labels":
        input_label = np.max(datap["segmentation"][seeds == input_seeds_label2])

    vessels = ima.select_labels(datap["segmentation"], input_label, slab=datap["slab"])

    # if type(input_label) is str:
    #     numeric_label = datap['slab'][input_label]
    # else:
    #     numeric_label = input_label
    # vessels = datap['segmentation'] == numeric_label

    vesselstmp = vessels
    sumall = np.sum(vessels == 1)

    # split_obj = scipy.ndimage.binary_dilation(split_obj, iterations = 5 )
    # vesselstmp = vessels * (1 - split_obj)

    lab, n_obj = scipy.ndimage.label(vesselstmp)
    logger.debug("number of objects " + str(n_obj))

    # while n_obj < 2 :
# dokud neni z celkoveho objektu ustipnuto alespon 80 procent
    not_complete = True
    while not_complete:
        if method == "reach volume":
            not_complete = np.sum(lab == qmisc.max_area_index(lab, n_obj)) > (vessel_volume_threshold * sumall)
        elif method == "separate labels":
            # misc.
            # imma.get_nlabel(datap["slab"], )
            # imma.select_labels(seeds,input_seeds_separate_label)
            seglab1 = np.max(lab[seeds == input_seeds_separate_label])
            seglab2 = np.max(lab[seeds == input_seeds_label2])
            if (seglab1 > 0) and (seglab2 > 0) and (seglab1 != seglab2):
                not_complete = False
        else:
            IOError("Unknown method " + str(method))

        split_obj = scipy.ndimage.binary_dilation(split_obj, iterations=dilatation_iterations)
        vesselstmp = vessels * (1 - split_obj)

        lab, n_obj = scipy.ndimage.label(vesselstmp)

    if method == "reach volume":
        # všechny objekty, na které se to rozpadlo
        # pyed = sed3.sed3(lab)
        # pyed.show()
        obj1 = get_biggest_object(lab)

    # vymaz nejvetsiho
        lab[obj1 == 1] = 0
        obj2 = get_biggest_object(lab)

        pixel = 0
        pixels = obj1[seeds == input_seeds_separate_label]
        if len(pixels) > 0:
            pixel = pixels[0]

        # from PyQt4.QtCore import pyqtRemoveInputHook
        # pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace() # BREAKPOINT

        if pixel > 0:
            ol1 = output_label1
            ol2 = output_label2
        else:
            ol2 = output_label1
            ol1 = output_label2

        # first selected pixel with right button
        lab = ol1 * obj1 + ol2 * obj2
    elif method == "separate labels":
        lab = (lab == seglab1) * output_label1 + (lab == seglab2) * output_label2
    cut_by_user = split_obj0
    return lab, cut_by_user


def Resekce_podle_bodu(data, seeds):
    lab, cut = split_vessel(data, seeds)
    segm, dist1, dist2 = split_organ_by_two_vessels(data, lab)
    data = virtual_resection_visualization(data, segm, dist1, dist2, cut)
    return data


def cut_editor(data, inputfile):
    # @TODO ošetřit modul viewer viz issue #69
    import viewer3
    # global normal,coordinates
    viewer = viewer3.Viewer(inputfile, 'View')
    # zobrazovani jater v kodu
    viewer.prohlizej(data, 'View', 'liver')

    # mesh = viewer.generate_mesh(segmentation,voxelsize_mm,degrad)
    # viewer.View(mesh,False)
    # viewer.buttons(window,grid)
    # print(viewer.normal)
    # print(viewer.coordinates)

    '''
    Funkce vrací trojrozměrné porobné jako data['segmentation']
    v data['slab'] je popsáno, co která hodnota znamená
    labels = []
    segmentation = segmentation[::degrad,::degrad,::degrad]
    print("Generuji data...")
    segmentation = segmentation[:,::-1,:]
    mesh_data = seg2fem.gen_mesh_from_voxels_mc(segmentation,
        voxelsize_mm*degrad)
    print("Done")
    if True:
        mesh_data.coors = seg2fem.smooth_mesh(mesh_data)
    vtk_file = "mesh_geom.vtk"
    mesh_data.write(vtk_file)
    app = QApplication(sys.argv)
    #view = viewer3.QVTKViewer(vtk_file,'Cut')
    '''

    # normal = viewer3.normal_and_coordinates().set_normal()
    # coordinates = viewer3.normal_and_coordinates().set_coordinates()
    # return normal,coordinates
    pass


def change(data, name):
    # data['segmentation'][vessels == 2] = data['slab']['porta']
    segmentation = data['segmentation']
    cut_editor(segmentation == data['slab'][name])

def velikosti(a):
    # a_index = [0, 0, 0]
    # for x in range(0, len(a)):
    #     for y in range(0, len(a[0])):
    #         for z in range(0, len(a[0][0])):
    #             if a[x][y][z] == 1:
    #                 a_index[0] += 1
    #             elif a[x][y][z] == 2:
    #                 a_index[1] += 1
    #             elif a[x][y][z] == 3:
    #                 a_index[2] += 1
    mx = np.max(a)
    a_index = []
    for i in range(1, 4): # for i in range(1, mx + 1):
        sm = np.sum(a == i)
        a_index.append(sm)

    return a_index

def nejnizsi(a, b, c):
    if a > b:
        if b > c:
            return 3
        else:
            return 2
    elif b > c:
        if c > a:
            return 1
        else:
            return 3
    elif c > a:
        if a > b:
            return 2
        else:
            return 1
    else:
        print("chyba")


def resection_portal_vein_new(data, interactivity=False, seeds=None, organ_label=1, vein_label=2):
    """
    New function for portal vein segmentation
    :param data:
    :param interactivity:
    :param seeds:
    :param kwargs:
    :return:
    """
    # ed = sed3.sed3(a)
    # ed.show()

    # from PyQt4 import QtGui
    # from PyQt4.QtGui import QApplication, QMainWindow, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QFrame, \
    # QFont, QPixmap, QFileDialog
    #
    # window = QtGui.QWidget()
    # mainLayout = QVBoxLayout()
    # window.setLayout(mainLayout)
    # mainLayout.addWidget(sed3.sed3qtWidget(data['data3d'], contour=data['segmentation']))

    # zachovani puvodnich dat
    segmentation = data["segmentation"]
    data3d = data["data3d"]

    # data pouze se segmentacemi
    segm = ((data["segmentation"] == organ_label) * organ_label +
            (data["segmentation"] == vein_label) * vein_label)


    # ed = sed3.sed3(segm)
    # ed.show()

    # ufiknutí segmentace
    crinfo = qmisc.crinfo_from_specific_data(segm, [0])
    data["segmentation"] = qmisc.crop(segm, crinfo)
    data["data3d"] = qmisc.crop(data3d, crinfo)
    if seeds is not None:
        seeds = qmisc.crop(seeds, crinfo)

    # @TODO zde nahradit střeve čímkoliv smysluplnějším
    if interactivity:
        print("Select cut")
        # seeds = cut_editor_old(data)
        seeds = cut_editor_old(data)
    elif seeds is None:
        logger.error('seeds is None and interactivity is False')
        return None

    lab, cut = split_vessel(data, seeds)
    segm, dist1, dist2 = split_organ_by_two_vessels(data, lab)

    # jatra rozdeleny na 3 kusy
    a = morphology.label(segm, background=0)
    ### podmínka nefunguje
    if 3 in a: # zda se v segmentaci objevuje 3. cast
        print("slape :) :) :P")
        a_index = velikosti(segm)
        print(a_index)
        i = nejnizsi(a_index[0], a_index[1], a_index[2])
        segm = ((a == i) * (segm == 1).astype('int8') +
                (a != i)*(segm == 2).astype('int8') +
                (segm != 0).astype('int8'))

    # TODO split this function from visualization
    data = virtual_resection_visualization(data, segm, dist1,
                                           dist2, cut,
                                           interactivity=interactivity)

    # vrácení původních dat a spojení s upravenými daty
    data["data3d"] = data3d
    # orig_shape = (len(segmentation), len(segmentation[0]), len(segmentation[1]))
    data["segmentation"] = qmisc.uncrop(data["segmentation"], crinfo, orig_shape=segmentation.shape)

    #segmentation = segmentation == vein
    data["segmentation"] = (data["segmentation"] +
                            (segmentation != organ_label) * segmentation) - (segmentation == vein_label) * vein_label
    return data

def resection_old(data, interactivity=True, seeds=None):
    if interactivity:
        print("Select cut")
        seeds = cut_editor_old(data)
    elif seeds is None:
        logger.error('seeds is None and interactivity is False')
        return None

    logger.debug("unique(seeds) " + str(np.unique(seeds)))
    # seeds[56][60][78] = 1
    lab, cut = split_vessel(data, seeds)
    segm, dist1, dist2 = split_organ_by_two_vessels(data, lab)
# TODO split this function from visualization
    data = virtual_resection_visualization(data, segm, dist1,
                                           dist2, cut,
                                           interactivity=interactivity)
    return data

def resection_planar(data, interactivity, seeds=None):
    """
    Based on input seeds the cutting plane is constructed

    :param data:
    :param interactivity:
    :param seeds:
    :return:
    """
    if seeds is None:
        if interactivity:
            print("Select cut")
            seeds = cut_editor_old(data)
        else:
            logger.error('seeds is None and interactivity is False')
            return None

    segm, dist1, dist2 = split_organ_by_plane(data, seeds)
    cut = dist1**2 < 2
    # TODO split this function from visualization
    data = virtual_resection_visualization(data, segm, dist1,
                                           dist2, cut,
                                           interactivity=interactivity)
    return data

def split_organ_by_plane(data, seeds):
    """
    Based on seeds split nonzero segmentation with plane

    :param data:
    :param seeds:
    :return:
    """
    from . import geometry3d
    from . import data_manipulation
    l1 = 1
    l2 = 2

    point, vector = geometry3d.plane_fit(seeds.nonzero())
    dist1 = data_manipulation.split_with_plane(point, vector, data['data3d'].shape)
    dist2 = dist1 * -1

    segm = (((data['segmentation'] != 0) * (dist1 < dist2)).astype('int8') +
            (data['segmentation'] != 0).astype('int8'))

    return segm, dist1, dist2


def split_tissue_on_labeled_tree(labeled_branches,
                                 trunk_label, branch_labels,
                                 tissue_segmentation, neighbors_list=None,
                                 ignore_labels=None,
                                 ignore_trunk=True,
                                 on_missed_branch="split",

                                 ):
    """
    Based on pre-labeled vessel tree split surrounding tissue into two part.
    The connected sub tree is computed and used internally.

    :param labeled_branches: ndimage with labeled volumetric vessel tree.
    :param trunk_label: int
    :param branch_labels: list of ints
    :param tissue_segmentation: ndimage with bool type. Organ is True, the rest is False.
    :param ignore_trunk: True or False
    :param ignore_labels: list of labels which will be ignored
    :param on_missed_branch: str, ["split", "organ_label", exception]. Missed label is label directly connected
    to trunk but with no branch label inside.
    "split" will ignore mised label.
    "orig" will leave the original area label.
    "exception", will throw the exception.
    :return:
    """
    # bl = lisa.virtual_resection.branch_labels(oseg, "porta")

    import imma.measure
    import imma.image_manipulation
    import imma.image_manipulation as ima

    if ignore_labels is None:
        ignore_labels = []

    ignore_labels = list(ignore_labels)
    if ignore_trunk:
        ignore_labels.append(trunk_label)

    if neighbors_list is None:
        exclude = [0]
        exclude.extend(ignore_labels)
        neighbors_list = imma.measure.neighbors_list(
            labeled_branches,
            None,
            # [seglabel1, seglabel2, seglabel3],
            exclude=exclude)
    #exclude=[imma.image_manipulation.get_nlabels(slab, ["liver"]), 0])
    # ex
    # print(neighbors_list)
    # find whole branche
    # segmentations = [None] * len(branch_labels)
    segmentation = np.zeros_like(labeled_branches, dtype=int)
    new_branches = []
    connected = [None] * len(branch_labels)

    for i, branch_label in enumerate(branch_labels):
        import copy

        ignore_other_branches = copy.copy(branch_labels)
        ignore_other_branches.pop(i)
        ignore_labels_i = [0]
        ignore_labels_i.extend(ignore_other_branches)
        ignore_labels_i.extend(ignore_labels)
        connected_i = imma.measure.get_connected_labels(
            neighbors_list, branch_label, ignore_labels_i)
        # segmentations[i] = ima.select_labels(labeled_branches, connected_i).astype(np.int8)
        select = ima.select_labels(labeled_branches, connected_i).astype(np.int8)
        select = select > 0
        if np.max(segmentation[select]) > 0:
            logger.debug("Missing branch connected to branch and other branch or trunk.")
            union = (segmentation * select) > 0
            segmentation[select] = i + 1
            if on_missed_branch == "split":
                segmentation[union] = 0
            elif on_missed_branch == "orig":
                new_branche_label = len(branch_labels) + len(new_branches) + 1
                logger.debug("new branch label {}".format(new_branche_label))
                segmentation[union] = new_branche_label
                new_branches.append(new_branche_label)
            elif on_missed_branch == "exception":
                raise ValueError("Missing one vessel")
            else:
                raise ValueError("Unknown 'on_missed_label' parameter.")
        else:
            segmentation[select] = i + 1
        # error
        # else:
        #   segmentation[select] = i + 1
        connected[i] = connected_i
    seg = segmentation
    # if np.max(np.sum(segmentations, 0)) > 1:
    #     raise ValueError("Missing one vessel")
    #
    # for i, branch_label in enumerate(branch_labels):
    #     segmentations[i] = segmentations[i] * (i + 1)
    # seg = np.sum(segmentations, 0)

    # ignore_labels1 = [0, trunk_label, branch_label2]
    # ignore_labels1.extend(ignore_labels)
    # ignore_labels2 = [0, trunk_label, branch_label]
    # ignore_labels2.extend(ignore_labels)
    # connected2 = imma.measure.get_connected_labels(
    #     neighbors_list, branch_label, ignore_labels1)
    # connected3 = imma.measure.get_connected_labels(
    #     neighbors_list, branch_label2, ignore_labels2)
    #
    # # seg = ima.select_labels(segmentation, organ_label, slab).astype(np.int8)
    # seg1 = ima.select_labels(labeled_branches, connected2).astype(np.int8)
    # seg2 = ima.select_labels(labeled_branches, connected3).astype(np.int8)
    # seg = seg1 + seg2 * 2
    # if np.max(seg) > 2:
    #     ValueError("Missing one vessel")

    dseg = ima.distance_segmentation(seg)
    logger.debug("output unique labels {}".format(np.unique(dseg)))
    # organseg = ima.select_labels(segmentation, organ_label, slab).astype(np.int8)
    dseg[~tissue_segmentation.astype(np.bool)] = 0

    return dseg, connected


def split_organ_by_two_vessels(datap,
                               seeds, organ_label=1,
                               seed_label1=1, seed_label2=2,
                               weight1=1, weight2=1):
    """

    Input of function is ndarray with 2 labeled vessels and data.
    Output is segmented organ by vessls using minimum distance criterium.

    :param datap: dictionary with 3d data, segmentation, and other information
           "data3d": 3d-ndarray with intensity data
           "voxelsize_mm",
           "segmentation": 3d ndarray with image segmentation
           "slab": segmentation labels
    :param seeds: ndarray with same size as data3d
            1: first part of portal vein (or defined in seed1_label)
            2: second part of portal vein (or defined in seed2_label)
    :param weight1: distance weight from seed_label1
    :param weight2: distance weight from seed_label2

    """
    weight1 = 1 if weight1 is None else weight1

    slab = datap["slab"]
    segmentation = datap["segmentation"]
    if type(seed_label1) != list:
        seed_label1 = [seed_label1]
    if type(seed_label2) != list:
        seed_label2 = [seed_label2]
    # dist se tady počítá od nul jenom v jedničkách
    dist1 = scipy.ndimage.distance_transform_edt(
        1 - ima.select_labels(seeds, seed_label1, slab),
        # seeds != seed_label1,
        sampling=datap['voxelsize_mm']
    )
    dist2 = scipy.ndimage.distance_transform_edt(
        1 - ima.select_labels(seeds, seed_label2, slab),
        # seeds != seed_label2,
        sampling=datap['voxelsize_mm']
    )
    # import skfmm
    # dist1 = skfmm.distance(
    #     labeled != l1,
    #     dx=datap['voxelsize_mm']
    # )
    # dist2 = skfmm.distance(
    #     labeled != l2,
    #     dx=datap['voxelsize_mm']
    # )
    # print 'skfmm'
    # from PyQt4.QtCore import pyqtRemoveInputHook; pyqtRemoveInputHook()
    # import ipdb; ipdb.set_trace()

    # from PyQt4.QtCore import pyqtRemoveInputHook
    # pyqtRemoveInputHook()
    # import ipdb; ipdb.set_trace() # BREAKPOINT

    # segm = (dist1 < dist2) * (data['segmentation'] != data['slab']['none'])
    target_organ_segmentation = ima.select_labels(segmentation, organ_label, slab)
    segm = ((target_organ_segmentation * ((dist1 / weight1) > (dist2 / weight2))).astype('int8') +
            target_organ_segmentation.astype('int8'))

    return segm, dist1, dist2


def virtual_resection_visualization(data, segm, dist1, dist2, cut,
                                    interactivity=True):
    v1, v2 = liver_spit_volume_mm3(segm, data['voxelsize_mm'])

    if interactivity:
        print("Liver volume: %.4g l" % ((v1 + v2) * 1e-6))
        print("volume1: %.4g l  (%.3g %%)" % (
            (v1) * 1e-6, 100 * v1 / (v1 + v2)))
        print("volume2: %.4g l  (%.3g %%)" % (
            (v2) * 1e-6, 100 * v2 / (v1 + v2)))

    # pyed = sed3.sed3(segm)
    # pyed.show()
    # import pdb; pdb.set_trace()
    linie = (((data['segmentation'] != 0) *
              (np.abs(dist1 - dist2) < 1))).astype(np.int8)
    linie_vis = 2 * linie
    linie_vis[cut == 1] = 1
    linie_vis = linie_vis.astype(np.int8)
    if interactivity:
        pyed = sed3.sed3qt(
            data['data3d'],
            seeds=linie_vis,
            contour=(data['segmentation'] != 0))
        # pyed.show()
        pyed.exec_()

    # import pdb; pdb.set_trace()

    # show3.show3(data['segmentation'])

    slab = {
        'liver': 1,
        'porta': 2,
        'resected_liver': 3,
        'resected_porta': 4}

    slab.update(data['slab'])

    data['slab'] = slab

    data['slab']['resected_liver'] = 3
    data['slab']['resected_porta'] = 4

    mask_resected_liver = (
        (segm == 1) & (data['segmentation'] == data['slab']['liver']))
    mask_resected_porta = (
        (segm == 1) & (data['segmentation'] == data['slab']['porta']))

    data['segmentation'][mask_resected_liver] = \
        data['slab']['resected_liver']
    data['segmentation'][mask_resected_porta] = \
        data['slab']['resected_porta']

    logger.debug('resection_old() end')
    return data


def resection_with_3d_visualization(data, name):

    # data['segmentation'][vessels == 2] = data['slab']['porta']
    # segmentation = data['segmentation']
    # print(data['slab'])
    change(data, name)

    # print data["slab"]
    # change(segmentation == data['slab']['porta'])
    # lab = cut_editor(segmentation == data['slab']['porta'])


def get_biggest_object(data):
    return qmisc.get_one_biggest_object(data)


def liver_spit_volume_mm3(segm, voxelsize_mm):
    """
    segm: 0 - nothing, 1 - remaining tissue, 2 - resected tissue
    """
    voxelsize_mm3 = np.prod(voxelsize_mm)
    v1 = np.sum(segm == 1) * voxelsize_mm3
    v2 = np.sum(segm == 2) * voxelsize_mm3

    return v1, v2


def View(name):
    data = misc.obj_from_file("out", filetype='pickle')
    resection(data, name)


def label_volumetric_vessel_tree(oseg, vessel_label=None, write_to_oseg=True, new_label_str_format="{}{:03d}"):
    """
    Split vessel by branches and put it in segmentation and slab.

    :param oseg: OrganSegmentation object with segmentation, voxelsize_mm and slab
    :param vessel_label: int or string label with vessel. Everything above zero is used if vessel_label is set None.
    :param write_to_oseg: Store output into oseg.segmentation if True. The slab is also updated.
    :param new_label_str_format: format of new slab
    :return:
    """
    logger.debug("vessel_label {}".format(vessel_label))
    logger.debug("python version {} {}".format(sys.version_info, sys.executable))
    import skelet3d
    if vessel_label is None:
        vessel_volume = oseg.segmentation > 0
    else:
        vessel_volume = oseg.select_label(vessel_label)

    # print(np.unique(vessel_volume))
    skel = skelet3d.skelet3d(vessel_volume)
    skan = skelet3d.SkeletonAnalyser(skel, volume_data=vessel_volume)
    skan.skeleton_analysis()
    bl = skan.get_branch_label()
    un = np.unique(bl)
    if write_to_oseg:
        for lb in un:
            if lb != 0:
                new_slabel = new_label_str_format.format(vessel_label, lb)
                new_nlabel = oseg.nlabels(new_slabel)
                oseg.segmentation[bl == lb] = new_nlabel

    # ima.distance_segmentation(oseg.select_label(vessel_label))
    return bl


if __name__ == "__main__":
    # logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

#    SectorDisplay2__()
    # logger.debug('input params')
    # input parser
    parser = argparse.ArgumentParser(description='Segment vessels from liver')
    parser.add_argument('-pkl', '--picklefile',
                        help='input file from organ_segmentation')
    parser.add_argument('-oe', '--use_old_editor',  action='store_true',
                        help='use an old editor for vessel cut')
    parser.add_argument('-o', '--outputfile',  default=None,
                        help='output file')
    parser.add_argument('-oo', '--defaultoutputfile',  action='store_true',
                        help='"vessels.pickle" as output file')
    parser.add_argument('-d', '--debug',  action='store_true',
                        help='Debug mode')
    args = parser.parse_args()

    if (args.picklefile or args.vtkfile) is None:
        raise IOError('No input data!')

    data = misc.obj_from_file(args.picklefile, filetype='pickle')
    ds = data['segmentation'] == data['slab']['liver']
    pozice = np.where(ds == 1)
    a = pozice[0][0]
    b = pozice[1][0]
    c = pozice[2][0]
    ds = False
    # print "vs ", data['voxelsize_mm']
    # print "vs ", data['voxelsize_mm']
    if args.debug:
        logger.setLevel(logging.DEBUG)
    # seg = np.zeros([100,100,100])
    # seg [50:80, 50:80, 60:75] = 1
    # seg[58:60, 56:72, 66:68]=2
    # dat = np.random.rand(100,100,100)
    # dat [50:80, 50:80, 60:75] =  dat [50:80, 50:80, 60:75] + 1
    # dat [58:60, 56:72, 66:68] =  dat  [58:60, 56:72, 66:68] + 1
    # slab = {'liver':1, 'porta':2, 'portaa':3, 'portab':4}
    # data = {'segmentation':seg, 'data3d':dat, 'slab':slab}
    name = 'porta'

    # cut_editor(data,args.inputfile)
    if args.use_old_editor:
        resection(data, name, method=args.use_old_editor)
    else:
        cut_editor(data, args.picklefile)
    # print normal
    # print coordinates

    defaultoutputfile = "05-resection.pkl"
    if args.defaultoutputfile:
        args.outputfile = defaultoutputfile

    if args.outputfile is None:

        savestring = raw_input('Save output data? (y/n): ')
        if savestring in ['Y', 'y']:

            misc.obj_to_file(data, defaultoutputfile, filetype='pickle')
    else:
        misc.obj_to_file(data, args.outputfile, filetype='pickle')



#! /usr/bin/python
# -*- coding: utf-8 -*-


import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/sed3"))

import numpy as np

import logging
logger = logging.getLogger(__name__)

import subprocess
import scipy
import scipy.ndimage

from imma.image_manipulation import crinfo_from_specific_data, crop, uncrop, manualcrop, fix_crinfo, combinecrinfo


class SparseMatrix():
    def __init__(self, ndarray):
        self.coordinates = ndarray.nonzero()
        self.shape = ndarray.shape
        self.values = ndarray[self.coordinates]
        self.dtype = ndarray.dtype
        self.sparse = True

    def todense(self):
        dense = np.zeros(self.shape, dtype=self.dtype)
        dense[self.coordinates[:]] = self.values
        return dense


def isSparseMatrix(obj):
    if obj.__class__.__name__ == 'SparseMatrix':
        return True
    else:
        return False


# import sed3

# def manualcrop(data):  # pragma: no cover
#
#     try:
#         from imcut import seed_editor_qt
#     except:
#         logger.warning("Deprecated of pyseg_base as submodule")
#         import seed_editor_qt
#
#     pyed = seed_editor_qt.QTSeedEditor(data, mode='crop')
#     pyed.exec_()
#     # pyed = sed3.sed3(data)
#     # pyed.show()
#     nzs = pyed.seeds.nonzero()
#     crinfo = [
#         [np.min(nzs[0]), np.max(nzs[0])],
#         [np.min(nzs[1]), np.max(nzs[1])],
#         [np.min(nzs[2]), np.max(nzs[2])],
#         ]
#     data = crop(data, crinfo)
#     return data, crinfo
#
#
# def crop(data, crinfo):
#     """
#     Crop the data.
#
#     crop(data, crinfo)
#
#     :param crinfo: min and max for each axis
#
#     """
#     crinfo = fix_crinfo(crinfo)
#     return data[
#         __int_or_none(crinfo[0][0]):__int_or_none(crinfo[0][1]),
#         __int_or_none(crinfo[1][0]):__int_or_none(crinfo[1][1]),
#         __int_or_none(crinfo[2][0]):__int_or_none(crinfo[2][1])
#         ]
#
#
# def __int_or_none(number):
#     if number is not None:
#         number = int(number)
#     return number


# def combinecrinfo(crinfo1, crinfo2):
#     """
#     Combine two crinfos. First used is crinfo1, second used is crinfo2.
#     """
#     crinfo1 = fix_crinfo(crinfo1)
#     crinfo2 = fix_crinfo(crinfo2)
#
#     crinfo = [
#         [crinfo1[0][0] + crinfo2[0][0], crinfo1[0][0] + crinfo2[0][1]],
#         [crinfo1[1][0] + crinfo2[1][0], crinfo1[1][0] + crinfo2[1][1]],
#         [crinfo1[2][0] + crinfo2[2][0], crinfo1[2][0] + crinfo2[2][1]]
#         ]
#
#     return crinfo
#
#
# def crinfo_from_specific_data(data, margin):
#     # hledáme automatický ořez, nonzero dá indexy
#     logger.debug('crinfo')
#     logger.debug(str(margin))
#     nzi = np.nonzero(data)
#     logger.debug(str(nzi))
#
#     x1 = np.min(nzi[0]) - margin[0]
#     x2 = np.max(nzi[0]) + margin[0] + 1
#     y1 = np.min(nzi[1]) - margin[0]
#     y2 = np.max(nzi[1]) + margin[0] + 1
#     z1 = np.min(nzi[2]) - margin[0]
#     z2 = np.max(nzi[2]) + margin[0] + 1
#
# # ošetření mezí polí
#     if x1 < 0:
#         x1 = 0
#     if y1 < 0:
#         y1 = 0
#     if z1 < 0:
#         z1 = 0
#
#     if x2 > data.shape[0]:
#         x2 = data.shape[0] - 1
#     if y2 > data.shape[1]:
#         y2 = data.shape[1] - 1
#     if z2 > data.shape[2]:
#         z2 = data.shape[2] - 1
#
# # ořez
#     crinfo = [[x1, x2], [y1, y2], [z1, z2]]
#     return crinfo
#
#
# def uncrop(data, crinfo, orig_shape, resize=False):
#
#     crinfo = fix_crinfo(crinfo)
#     data_out = np.zeros(orig_shape, dtype=data.dtype)
#
#     # print 'uncrop ', crinfo
#     # print orig_shape
#     # print data.shape
#     if resize:
#         data = resize_to_shape(data, crinfo[:, 1] - crinfo[:, 0])
#
#     startx = np.round(crinfo[0][0]).astype(int)
#     starty = np.round(crinfo[1][0]).astype(int)
#     startz = np.round(crinfo[2][0]).astype(int)
#
#     data_out[
#         # np.round(crinfo[0][0]).astype(int):np.round(crinfo[0][1]).astype(int)+1,
#         # np.round(crinfo[1][0]).astype(int):np.round(crinfo[1][1]).astype(int)+1,
#         # np.round(crinfo[2][0]).astype(int):np.round(crinfo[2][1]).astype(int)+1
#         startx:startx + data.shape[0],
#         starty:starty + data.shape[1],
#         startz:startz + data.shape[2]
#         ] = data
#
#     return data_out
#
#
# def fix_crinfo(crinfo, to='axis'):
#     """
#     Function recognize order of crinfo and convert it to proper format.
#     """
#
#     crinfo = np.asarray(crinfo)
#     if crinfo.shape[0] == 2:
#         crinfo = crinfo.T
#
#     return crinfo


# def getVersionString():
#     """
#     Function return string with version information.
#     It is performed by use one of three procedures: git describe,
#     file in .git dir and file __VERSION__.
#     """
#     version_string = None
#     try:
#         version_string = subprocess.check_output(['git', 'describe'])
#         if sys.version_info.major == 3:
#             version_string = version_string.decode("utf8")
#     except:
#         logger.warning('Command "git describe" is not working')
#
#     if version_string == None:  # noqa
#         try:
#             path_to_version = os.path.join(path_to_script,
#                                            '../.git/refs/heads/master')
#             with file(path_to_version) as f:
#                 version_string = f.read()
#         except:
#             logger.warning('Problem with reading file ".git/refs/heads/master"')
#
#     if version_string == None:  # noqa
#         try:
#             path_to_version = os.path.join(path_to_script, '../__VERSION__')
#             with file(path_to_version) as f:
#                 version_string = f.read()
#             path_to_version = path_to_version + \
#                 '  version number is created manually'
#
#         except:
#             logger.warning('Problem with reading file "__VERSION__"')
#
#     # if version_string is None:
#     #     version_string = ""
#     return version_string


def get_one_biggest_object(data):
    """ Return biggest object """
    lab, num = scipy.ndimage.label(data)
    # print("bum = "+str(num))

    maxlab = max_area_index(lab, num)

    data = (lab == maxlab)
    return data


def max_area_index(labels, num):
    """
    Return index of maxmum labeled area
    """
    mx = 0
    mxi = -1
    for l in range(1, num + 1):
        mxtmp = np.sum(labels == l)
        if mxtmp > mx:
            mx = mxtmp
            mxi = l

    return mxi

from io3d.misc import resize_to_mm, resize_to_shape


# def resize_to_shape(*pars, **params):
#     import io3d.misc
#     return io3d.misc.resize_to_shape(*pars, **params)



#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Module is used for visualization of segmentation stored in pkl file.
"""

import sys
import os.path
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/dicom2fem/src"))
import logging
logger = logging.getLogger(__name__)

# from PyQt4.QtCore import Qt
from PyQt4.QtGui import QApplication
import argparse

import vtk

import numpy as np
from dicom2fem import seg2fem
# import misc
import viewer
import io3d
import dicom2fem
from imtools.image_manipulation import select_labels


def seg2stl(
        segmentation,
        voxelsize_mm=np.ones([3, 1]),
        degrad=4,
        labels=[1],
        smoothing=True,
        outputfile="output.stl",
        tempfile="mesh_geom.vtk",
        ):
    """
    Funkce vrací trojrozměrné porobné jako data['segmentation']
    v data['slab'] je popsáno, co která hodnota znamená
    """
    print(np.unique(segmentation))
    segmentation = select_labels(segmentation, labels)

    # print 'labels: ', np.unique(data['segmentation'])
    # print np.sum(data['segmentation'] == 0)
    # print args.labels
    # for i in range(0, len(args.label)):

    segmentation = segmentation[::degrad, ::degrad, ::degrad]

    print(np.unique(segmentation))

    # import pdb; pdb.set_trace()
    if smoothing:
        mesh_data = seg2fem.gen_mesh_from_voxels(segmentation, voxelsize_mm*degrad*1e-3, etype='t', mtype='s')
        mesh_data.coors = dicom2fem.seg2fem.smooth_mesh(mesh_data)
    else:
        mesh_data = dicom2fem.seg2fem.gen_mesh_from_voxels_mc(segmentation, voxelsize_mm * degrad * 1.0e-2)
        # mesh_data.coors +=
    mesh_data.write(tempfile)
    dicom2fem.vtk2stl.vtk2stl(tempfile, outputfile)
    # QApplication(sys.argv)
    # view = viewer.QVTKViewer(vtk_file)
    # view.exec_()


if __name__ == "__main__":
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
        description='\
            convert segmentation stored in pklz file into stl\n\
            \npython convert.py -i resection.pkl -l 2 3 4 -d 4')
    parser.add_argument(
        '-i', '--inputfile',
        default='organ.pkl',
        help='input file')
    parser.add_argument(
        '-o', '--outputfile',
        default='output.stl',
        help='output file')
    parser.add_argument(
        '-t', '--tempfile',
        default='mesh_geom.vtk',
        help='temp file used in processing')
    parser.add_argument(
        '-d', '--degrad', type=int,
        default=4,
        help='data degradation, default 4')
    parser.add_argument(
        '-l', '--labels', type=int, metavar='N', nargs='+',
        default=[1],
        help='segmentation labels, default 1')
    parser.add_argument(
        '-s', '--show', action='store_true',
        help='Show mode')
    args = parser.parse_args()

    dr = io3d.DataReader()
    data = dr.Get3DData(args.inputfile, dataplus_format=True)
    # args.label = np.array(eval(args.label))
    # print args.label
    # import pdb; pdb.set_trace()
    ds = data['segmentation']

    if args.show:
        dsel = select_labels(ds, args.labels)
        import sed3
        ed = sed3.sed3(dsel.astype(np.double))
        ed.show()


    seg2stl(ds, labels=args.labels, degrad=args.degrad, outputfile=args.outputfile, tempfile=args.tempfile)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR%  <>
#
# Distributed under terms of the %LICENSE% license.

"""

"""

import logging

logger = logging.getLogger(__name__)
import numpy as np
import argparse
import scipy

import imtools

from . import organ_model
from . import data_manipulation


def automatic_liver_seeds(
        data3d,
        seeds,
        voxelsize_mm,
        fn_mdl='~/lisa_data/liver_intensity.Model.p',
        return_likelihood_difference=True,
        gaussian_sigma_mm=[20,20,20]):
    from imcut import pycut
    # fn_mdl = op.expanduser(fn_mdl)
    mdl = pycut.Model({'mdl_stored_file':fn_mdl, 'fv_extern': organ_model.intensity_localization_fv})
    working_voxelsize_mm = np.asarray([1.5, 1.5, 1.5])
    gaussian_sigma_mm = np.asarray(gaussian_sigma_mm)

    data3dr = imtools.resize_to_mm(data3d, voxelsize_mm, working_voxelsize_mm)

    lik1 = mdl.likelihood_from_image(data3dr, working_voxelsize_mm, 0)
    lik2 = mdl.likelihood_from_image(data3dr, working_voxelsize_mm, 1)

    dl = lik2 - lik1

    seeds = add_negative_notrain_seeds(seeds,lik1, lik2)


    # Liver seed center
    import scipy

    # seed tam, kde je to nejpravděpodovnější - moc nefunguje při blbém natrénování
    dst = scipy.ndimage.filters.gaussian_filter(dl, sigma=gaussian_sigma_mm/working_voxelsize_mm)
    # seed1 = np.unravel_index(np.argmax(dl), dl.shape)

    # escte jinak
    # dáme seed doprostřed oblasti
    # dst = scipy.ndimage.morphology.distance_transform_edt(dl>0)

    seed1 = np.unravel_index(np.argmax(dst), dst.shape)
    # alternativa -
    seed1_mm = seed1 * working_voxelsize_mm
    print('seed1 ', seed1, ' shape ', dst.shape)

    seed1z = seed1[0]
    seed1z_mm = seed1_mm[0]
    print(seed1z_mm)


    add_negative_train_seeds_blobs(
        dl < 0,
        seeds,
        working_voxelsize_mm,
        voxelsize_mm,
        seed1z_mm, n_seed_blob=3)

    seeds = data_manipulation.add_seeds_mm(
        seeds, voxelsize_mm,
        [seed1_mm[0]],
        [seed1_mm[1]],
        [seed1_mm[2]],
        label=1,
        radius=25,
        width=1
    )
    # import sed3
    # sed3.show_slices(data3dr, dl > 40.0, slice_step=10)
    if return_likelihood_difference:
        return seeds, dl
    else:
        return seeds


def add_negative_notrain_seeds(seeds,lik1, lik2, alpha=1.3):
    """

    :param seeds:
    :param lik1:
    :param lik2:
    :param alpha: 1.2 means 20% to liver likelihood
    :return:
    """
    # dl = 2*lik2 - lik1
    dl = (alpha*lik1-lik2)>0
    # for sure we take two iterations from segmentation
    # dl[0,:,:] = True
    # dl[:,0,:] = True
    # dl[:,:,0] = True
    # dl[-1,:,:] = True
    # dl[:,-1,:] = True
    # dl[:,:,-1] = True
    dl = scipy.ndimage.morphology.binary_erosion(dl, iterations=2, border_value=1)

    # and now i will take just thin surface
    dl_before=dl
    dl = scipy.ndimage.morphology.binary_erosion(dl, iterations=2, border_value=1)
    dl = imtools.io3d.misc.resize_to_shape((dl_before - dl) > 0, seeds.shape)
    seeds[dl>0] = 4

    return seeds

def add_negative_train_seeds_blobs(
        mask_working,
        seeds,
        mask_voxelsize_mm,
        seeds_voxelsize_mm,
        seed1z_mm, n_seed_blob=1):
    dll = mask_working

    # aby se pocitalo i od okraju obrazku
    dll[0,:,:] = 0
    dll[:,0,:] = 0
    dll[:,:,0] = 0
    dll[-1,:,:] = 0
    dll[:,-1,:] = 0
    dll[:,:,-1] = 0
    for i in range(0, n_seed_blob):

        dst = scipy.ndimage.morphology.distance_transform_edt(dll)
        # na nasem rezu
        seed1z_px_mask = int(seed1z_mm/ mask_voxelsize_mm[0])
        seed1z_px_seeds= int(seed1z_mm/ seeds_voxelsize_mm[0])

        dstslice = dst[seed1z_px_mask, :, :]
        seed2xy = np.unravel_index(np.argmax(dstslice), dstslice.shape)
        # import PyQt4; PyQt4.QtCore.pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace()
        seed2 = np.array([seed1z_px_mask, seed2xy[0], seed2xy[1]])
        seed2_mm = seed2 * mask_voxelsize_mm

        seeds = data_manipulation.add_seeds_mm(
                seeds, seeds_voxelsize_mm,
                [seed2_mm[0]],
                [seed2_mm[1]],
                [seed2_mm[2]],
                label=2,
                radius=20,
                width=1
        )
        # for next iteration add hole where this blob is
        dll[seed1z_px_mask, seed2xy[0], seed2xy[1]] = 0

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)


if __name__ == "__main__":
    main()


# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Modul is used for GUI of Lisa
"""
import logging
logger = logging.getLogger(__name__)
# from lisa.logWindow import QVBoxLayout

import sys
import os
import numpy as np

import datetime
import functools

from io3d import datareader
# import segmentation

try:
    from viewer import QVTKViewer
    viewer3D_available = True

except ImportError:
    viewer3D_available = False

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/imcut/src"))

from PyQt4.QtGui import QApplication, QMainWindow, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QFrame, \
    QFont, QPixmap, QFileDialog, QInputDialog
from PyQt4 import QtGui

import sys
if sys.version_info.major == 2:
    from PyQt4.Qt import QString
else:
    Qstring = str

try:
    from imcut.seed_editor_qt import QTSeedEditor
except:
    logger.warning("Deprecated of pyseg_base as submodule")
    try:
        from imcut.seed_editor_qt import QTSeedEditor
    except:
        logger.warning("Deprecated of pyseg_base as submodule")
        from seed_editor_qt import QTSeedEditor

import sed3
from . import loginWindow
from . import dictEditQt
from . import segmentationQt
from . import virtual_resection
from io3d.network import download_file
from imtools.select_label_qt import SelectLabelWidget

def find_logo():
    logopath = os.path.join(path_to_script, "./icons/LISA256.png")
    if os.path.exists(logopath):
        return logopath
    # lisa runtime directory
    logopath = os.path.expanduser("~/lisa_data/.lisa/LISA256.png")
    if not os.path.exists(logopath):
        try:
            # wget.download(
            download_file(
                "https://raw.githubusercontent.com/mjirik/lisa/master/lisa/icons/LISA256.png",
                filename=logopath
            )
        except:
            logger.warning('logo download failed')
            pass
    if os.path.exists(logopath):
        return logopath

    pass


# GUI
class OrganSegmentationWindow(QMainWindow):

    def __init__(self, oseg=None, qapp=None):

        self.oseg = oseg
        self.qapp = qapp


        QMainWindow.__init__(self)
        self._initUI()
        self._initMenu()

        # if oseg is not None:
        #     if oseg.data3d is not None:
        #         self.setLabelText(self.text_dcm_dir, self.oseg.datapath)
        #         self.setLabelText(self.text_dcm_data, self.getDcmInfo())

        self.statusBar().showMessage('Ready')

    def _initMenu(self):
        menubar = self.menuBar()

        ###### FILE MENU ######
        fileMenu = menubar.addMenu('&File')
        loadSubmenu = fileMenu.addMenu('&Load')
        # load dir
        loadDirAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Directory', self)
        loadDirAction.setStatusTip('Load data from directory (DICOM, jpg, png...)')
        loadDirAction.triggered.connect(self.loadDataDir)
        loadSubmenu.addAction(loadDirAction)
        # load file
        loadFileAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&File', self)
        loadFileAction.setStatusTip('Load data from file (pkl, 3D Dicom, tiff...)')
        loadFileAction.triggered.connect(self.loadDataFile)
        loadSubmenu.addAction(loadFileAction) 
        
        saveSubmenu = fileMenu.addMenu('&Save')
        # save file
        saveFileAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&File', self)
        saveFileAction.setStatusTip('Save data with segmentation')
        saveFileAction.triggered.connect(self.saveOut)
        saveSubmenu.addAction(saveFileAction)   
        # save dicom
        saveDicomAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&DICOM', self)
        saveDicomAction.setStatusTip('Save DICOM data')
        saveDicomAction.triggered.connect(self.btnSaveOutDcm)
        saveSubmenu.addAction(saveDicomAction)
        # save dicom overlay
        saveDicomOverlayAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&DICOM overlay', self)
        saveDicomOverlayAction.setStatusTip('Save overlay DICOM data')
        saveDicomOverlayAction.triggered.connect(self.btnSaveOutDcmOverlay)
        saveSubmenu.addAction(saveDicomOverlayAction)     
        # save JSON
        saveJSONAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&JSON file', self)
        saveJSONAction.setStatusTip('Save JSON file')
        saveJSONAction.triggered.connect(self.btnSaveJSON)
        saveSubmenu.addAction(saveJSONAction)     
        # save PV tree
        savePVAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&PV tree', self)
        savePVAction.setStatusTip('Save Portal Vein 1D model')
        savePVAction.triggered.connect(self.btnSavePortalVeinTree)
        saveSubmenu.addAction(savePVAction)     
        # save HV tree
        saveHVAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Vessel Tree', self)
        saveHVAction.setStatusTip('Save actual vessel tree 1D model')
        saveHVAction.triggered.connect(self.btnSaveActualVesselTree)
        saveSubmenu.addAction(saveHVAction)     

        separator = fileMenu.addAction("")
        separator.setSeparator(True)

        autoSeedsAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Automatic liver seeds', self)
        autoSeedsAction.setStatusTip('Automatic liver seeds')
        autoSeedsAction.triggered.connect(self.btnAutomaticLiverSeeds)
        fileMenu.addAction(autoSeedsAction)

        autoSegAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Automatic segmentation', self)
        autoSegAction.setStatusTip('Automatic segmentation')
        autoSegAction.triggered.connect(self.btnAutoSeg)
        fileMenu.addAction(autoSegAction)

        cropAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Crop', self)
        cropAction.setStatusTip('')
        cropAction.triggered.connect(self.cropDcm)
        fileMenu.addAction(cropAction)

        maskAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Mask region', self)
        maskAction.setStatusTip('')
        maskAction.triggered.connect(self.maskRegion)
        fileMenu.addAction(maskAction)

        segFromFile = QtGui.QAction(QtGui.QIcon('exit.png'), '&Segmentation from file', self)
        segFromFile.setStatusTip('Load segmentation from pkl file, raw, ...')
        segFromFile.triggered.connect(self.btnLoadSegmentationFromFile)
        fileMenu.addAction(segFromFile)

        segFromOverlay = QtGui.QAction(QtGui.QIcon('exit.png'), '&Segmentation from Dicom overlay', self)
        segFromOverlay.setStatusTip('Load segmentation from Dicom file stack')
        segFromOverlay.triggered.connect(self.btnLoadSegmentationFromDicomOverlay)
        fileMenu.addAction(segFromOverlay)

        view3DAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&View 3D', self)
        view3DAction.setStatusTip('View segmentation in 3D model')
        view3DAction.triggered.connect(self.view3D)
        fileMenu.addAction(view3DAction)

        debugAction= QtGui.QAction(QtGui.QIcon('exit.png'), '&Debug terminal', self)
        debugAction.setStatusTip('Run interactive terminal debug')
        debugAction.triggered.connect(self.run_debug)
        fileMenu.addAction(debugAction)

        separator = fileMenu.addAction("")
        separator.setSeparator(True)


        exitAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Exit', self)
        exitAction.setShortcut('Ctrl+Q')
        exitAction.setStatusTip('Exit application')
        exitAction.triggered.connect(QtGui.qApp.quit)
        fileMenu.addAction(exitAction)


        ###### IMAGE MENU ######
        imageMenu = menubar.addMenu('&Image')

        randomRotateAction= QtGui.QAction(QtGui.QIcon('exit.png'), '&Random Rotate', self)
        # autoSeedsAction.setShortcut('Ctrl+Q')
        randomRotateAction.setStatusTip('Random rotation')
        randomRotateAction.triggered.connect(self.btnRandomRotate)
        imageMenu.addAction(randomRotateAction)

        mirrorZAxisAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Mirror Z-axis', self)
        mirrorZAxisAction.setStatusTip('Mirror Z-axis')
        mirrorZAxisAction.triggered.connect(self.oseg.mirror_z_axis)
        imageMenu.addAction(mirrorZAxisAction)

        segmentation_by_convex_areas = QtGui.QAction(QtGui.QIcon('exit.png'), '&Draw convex segmentation', self)
        segmentation_by_convex_areas.setStatusTip('Create segmentation by adding convex areas')
        segmentation_by_convex_areas.triggered.connect(self.action_add_segmentation_by_convex_areas)
        imageMenu.addAction(segmentation_by_convex_areas)

        segmentation_relabel_action = QtGui.QAction(QtGui.QIcon('exit.png'), '&Relabel segmentation', self)
        segmentation_relabel_action.setStatusTip('Change label of the segmentation')
        segmentation_relabel_action.triggered.connect(self.action_segmentation_relabel)
        imageMenu.addAction(segmentation_relabel_action)

        branch_label_action = QtGui.QAction(QtGui.QIcon('exit.png'), '&Label vessel tree', self)
        branch_label_action.setStatusTip('Label volumetric vessel tree')
        branch_label_action.triggered.connect(self.action_label_volumetric_vessel_tree)
        imageMenu.addAction(branch_label_action)

        resize_to_mm_action = QtGui.QAction(QtGui.QIcon('exit.png'), "Resize to mm", self)
        resize_to_mm_action.setStatusTip('Resize data3d and segemntation to mm')
        resize_to_mm_action.triggered.connect(self.action_resize_mm)
        imageMenu.addAction(resize_to_mm_action)

        self.__add_action_to_menu(imageMenu, "&Split tissue", self.action_split_on_bifurcation,
                                  tip="Split tissue based on labeled vessel tree",
                                  init_msg="Split tissue...",
                                  finish_msg="Ready. Tissue split finished."
                                  )

        self.__add_action_to_menu(imageMenu, "&Minimize slab", self.oseg.minimize_slab,
                                  tip="Remove unused or redundant labels from segmentation labeling list (slab)")

        self.__add_action_to_menu(imageMenu, "&Store seeds",
                                  lambda: self.oseg.save_seeds(self.ui_select_from_list(
                                      self.oseg.get_list_of_saved_seeds(), "Save seeds as")),
                                  tip="Save seeds for later use")

        self.__add_action_to_menu(imageMenu, "&Load seeds",
                                  lambda: self.oseg.load_seeds(self.ui_select_from_list(
                                      self.oseg.get_list_of_saved_seeds(), "Save seeds as")),
                                  tip="Save seeds for later use")

        self.__add_action_to_menu(imageMenu, "&Export seeds fo files",
                                  lambda: self.oseg.export_seeds_to_files(self.ui_select_output_filename(
                                      "mhd", window_title="Select pattern for seed files")),
                                  tip="Export seeds and stored seeds to files",
                                  finish_msg="Ready. Seeds exported."
                                  )


        self.__add_action_to_menu(imageMenu, "&Import seeds from file",
                                  lambda: self.oseg.import_seeds_from_file(self.ui_select_output_filename(
                                      "*", window_title="Select file with seeds")),
                                  tip="Import seeds",
                                  finish_msg="Ready. Seeds imported."
                                  )


        self.__add_action_to_menu(imageMenu, "&Body navigation structures", self.oseg.get_body_navigation_structures,
                                  tip="Put structures located by bodynavigation into segmentataion",
                                  init_msg="Get body navigation structures ...",
                                  finish_msg="Ready. Body navigation structures are in segmentation now."
                                  )

        self.__add_action_to_menu(imageMenu, "&Precise body navigation structures", self.oseg.get_body_navigation_structures_precise,
                                  tip="Put structures located by bodynavigation into segmentataion",
                                  init_msg="Get precise body navigation structures ...",
                                  finish_msg="Ready. Body navigation structures are in segmentation now."
                                  )

        ###### OPTION MENU ######
        optionMenu = menubar.addMenu('&Option')
        
        configAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Configuration', self)
        configAction.setStatusTip('Config settings')
        configAction.triggered.connect(self.btnConfig)
        optionMenu.addAction(configAction)

        editSlab = QtGui.QAction(QtGui.QIcon('exit.png'), '&Edit labels', self)
        editSlab.setStatusTip('Edit segmentation labels')
        editSlab.triggered.connect(self.btnEditSlab)
        optionMenu.addAction(editSlab)

        logAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Log', self)
        logAction.setStatusTip('See log file')
        logAction.triggered.connect(self.btnLog)
        optionMenu.addAction(logAction)

        syncAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Sync', self)
        syncAction.setStatusTip('Synchronize files from the server')
        syncAction.triggered.connect(self.sync_lisa_data)
        optionMenu.addAction(syncAction)

        unlockAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Unlock all buttons', self)
        unlockAction.setStatusTip('Unlock all locked buttons')
        unlockAction.triggered.connect(self.unlockAllButtons)
        optionMenu.addAction(unlockAction)

        updateAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&Update', self)
        updateAction.setStatusTip('Check new update')
        updateAction.triggered.connect(self.btnUpdate)
        optionMenu.addAction(updateAction)


        ###### CONFIG MENU ######
        configMenu = menubar.addMenu('&Config')
        # combo = QtGui.QComboBox(self)
        for text in self.oseg.segmentation_alternative_params.keys():
            iAction = QtGui.QAction(QtGui.QIcon('exit.png'), '&' + text, self)
            iAction.setStatusTip('Use predefined config "%s"' % (text))
            # something like lambda
            fn = functools.partial(self.onAlternativeSegmentationParams, text)
            iAction.triggered.connect(fn)

            configMenu.addAction(iAction)
        # combo.activated[str].connect(self.onAlternativeSegmentationParams)
        # grid.addWidget(combo, 4, 1)

    def __add_action_to_menu(self, submenu, ampersand_name, triggered_connect, tip='', init_msg=None, finish_msg=None):
        """

        :param submenu:
        :param ampersand_name: name with ampersand like '&Directory
        :param triggered_connect:
        :param tip:
        :return:
        """
        this_action = QtGui.QAction(QtGui.QIcon('exit.png'), ampersand_name, self)
        this_action.setStatusTip(tip)
        this_action.triggered.connect(lambda: self._ui_run_with(
            triggered_connect, init_msg=init_msg, finish_msg=finish_msg
        ))
        submenu.addAction(this_action)

    def _add_button(
            self,
            text,
            callback,
            uiw_label=None,
            tooltip=None,
            icon=None
                    ):
        if uiw_label is None:
            uiw_label = text
        btn = QPushButton(text, self)
        btn.clicked.connect(callback)
        if icon is not None:
            btn.setIcon(btn.style().standardIcon(icon))
        self.uiw[uiw_label] = btn
        if tooltip is not None:
            btn.setToolTip(tooltip)

        return btn

    def _activate_widget(self, widget):
        self.currentWidget.hide()
        self.grid.addWidget(widget,6,2)
        widget.show()
        self.currentWidget = widget

    def _initUI(self):
        window = QtGui.QWidget()
        self.window = window
        self.setCentralWidget(window)
        self.resize(800, 600)
        self.setWindowTitle('LISA')
        self.statusBar().showMessage('Ready')
        self.mainLayout = QHBoxLayout(window)
        window.setLayout(self.mainLayout)
        self.ui_widgets = {}
        self.ui_buttons = {}


        #### MENU ####
        self.initUIMenu()
        line = QFrame()
        line.setFrameShape(QFrame.VLine)
        self.mainLayout.addWidget(line)


        ##### BODY #####
        bodyLayout = QVBoxLayout()
        self.bodyLayout = bodyLayout
        self.mainLayout.addLayout(bodyLayout)

        #--- title ---
        self.infoBody = QtGui.QWidget()
        infoBodyLayout = QVBoxLayout()
        bodyLayout.addWidget(self.infoBody)
        self.infoBody.setLayout(infoBodyLayout)
        self.ui_widgets["Main"] = self.infoBody

        font_label = QFont()
        font_label.setBold(True)
        font_info = QFont()
        font_info.setItalic(True)
        font_info.setPixelSize(12)
        lisa_title = QLabel('Liver Surgery Analyser')
        if sys.version_info.major == 2:
            names = QString.fromUtf8('M. Jiřík, V. Lukeš - 2013')
        else:
            names = "M. Jiřík, V. Lukeš - 2013"
        info = QLabel('Developed by:\n' +
                      'University of West Bohemia\n' +
                      'Faculty of Applied Sciences\n' +
                      names +
                      '\n\nVersion: ' + self.oseg.version
                      )
        info.setFont(font_info)
        lisa_title.setFont(font_label)
        infoBodyLayout.addWidget(lisa_title)
        infoBodyLayout.addWidget(info)

        #--- segmentation option ---
        self.segBody = segmentationQt.SegmentationWidget(oseg=self.oseg, lisa_window=self)
        # self.segBody.oseg = self.oseg
        bodyLayout.addWidget(self.segBody)
        self.ui_widgets["Segmentation"] = self.segBody
        # self.segBody.lblSegData.setText(self.text_seg_data)
        self.segBody.btnVirtualResectionPV.clicked.connect(self.btnVirtualResectionPV)
        self.segBody.btnVirtualResectionPlanar.clicked.connect(self.btnVirtualResectionPlanar)
        self.segBody.btnVirtualResectionPV_testing.clicked.connect(self.btnVirtualResectionPV_new)


        ###
        self.segBody.btnSegManual.clicked.connect(self.btnManualSeg)
        self.segBody.btnSegSemiAuto.clicked.connect(self.btnSemiautoSeg)
        self.segBody.btnSegMask.clicked.connect(self.maskRegion)
        self.segBody.btnSegPV.clicked.connect(self.btnPortalVeinSegmentation)
        self.segBody.btnSegHV.clicked.connect(self.btnHepaticVeinsSegmentation)

        #--- edit slab ---
        self.slabBody = dictEditQt.DictEdit(dictionary=self.oseg)
        bodyLayout.addWidget(self.slabBody)
        self.ui_widgets["EditSlab"] = self.slabBody
        self.slabBody.btnSaveSlab.clicked.connect(self.segBody.reinitLabels)

        # -- load widget
        import io3d.datareaderqt
        self.read_widget = io3d.datareaderqt.DataReaderWidget(
            before_function=self._before_read_callback,
            after_function=self._after_read_callback,
            qt_app=self.qapp
        )
        self.read_widget.cache = self.oseg.cache
        bodyLayout.addWidget(self.read_widget)
        self.ui_widgets["Load"] = self.read_widget

        #--- file info (footer) ---
        bodyLayout.addStretch()

        self.oseg.gui_update = self.gui_update # nefunguje automaticky !!

        ##### OTHERS #####
        self.mainLayout.addStretch()

        self.btnSave.setDisabled(True)
        self.btnSegmentation.setDisabled(True)
        self.btnCompare.setDisabled(True)
        self.changeWidget('Main')
        self.unlockAllButtons()
        self.show()

    def gui_update(self):
        self.segBody.reinitLabels()
        self.slabBody.discardChanges()

    def initLogo(self, layout):
        font_label = QFont()
        font_label.setBold(True)
        font_info = QFont()
        font_info.setItalic(True)
        font_info.setPixelSize(12)

        lisa_logo = QLabel()
        logopath = find_logo()
        logo = QPixmap(logopath)
        logo = logo.scaled(130, 130)
        lisa_logo.setPixmap(logo)  # scaledToWidth(128))
        lisa_logo.mousePressEvent = lambda event: self.changeWidget('Main')
        layout.addWidget(lisa_logo)
        return layout

    def initUIMenu(self):
        ###### MAIN MENU ######
        self.mainLayoutRight = QVBoxLayout()
        mainLayoutRight = self.mainLayoutRight
        self.mainLayout.addLayout(mainLayoutRight)

        # ----- logo -----
        self.initLogo(mainLayoutRight)

        # --load--
        self.btnLoadWidget = QPushButton("Load", self)
        self.btnLoadWidget.clicked.connect(lambda: self.changeWidget('Load'))
        mainLayoutRight.addWidget(self.btnLoadWidget)

        # --save--
        self.btnSave = QPushButton("Save/Export")
        mainLayoutRight.addWidget(self.btnSave)

        saveFileAction = QtGui.QAction(QtGui.QIcon('exit.png'), "File", self)
        saveFileAction.triggered.connect(self.saveOut)

        saveDicomAction = QtGui.QAction(QtGui.QIcon('exit.png'), "Export Dicom", self)
        saveDicomAction.triggered.connect(self.btnSaveOutDcm)

        saveDicomOverlayAction = QtGui.QAction(QtGui.QIcon('exit.png'), "Export Dicom overlay", self)
        saveDicomOverlayAction.triggered.connect(self.btnSaveOutDcmOverlay)

        saveJSONAction = QtGui.QAction(QtGui.QIcon('exit.png'), "JSON file", self)
        saveJSONAction.triggered.connect(self.btnSaveJSON)

        saveImageStackAction = QtGui.QAction(QtGui.QIcon('exit.png'), "Image stack", self)
        saveImageStackAction.triggered.connect(self.saveOutImageStack)

        savePVTreeAction = QtGui.QAction(QtGui.QIcon('exit.png'), "PV Tree", self)
        savePVTreeAction.triggered.connect(self.btnSavePortalVeinTree)

        saveHVTreeAction = QtGui.QAction(QtGui.QIcon('exit.png'), "Vessel Tree", self)
        saveHVTreeAction.setStatusTip('Save actual vessel tree 1D model')
        saveHVTreeAction.triggered.connect(self.btnSaveActualVesselTree)


        menu = QtGui.QMenu(self.btnSave)
        menu.addAction(saveFileAction)
        menu.addAction(saveImageStackAction)
        menu.addAction(saveDicomAction)
        menu.addAction(saveDicomOverlayAction)
        menu.addAction(saveJSONAction)
        menu.addAction(savePVTreeAction)
        menu.addAction(saveHVTreeAction)
        self.btnSave.setMenu(menu)

        # --segmentation--
        self.btnSegmentation = QPushButton("Segmentation", self)
        self.btnSegmentation.setCheckable(True)
        self.btnSegmentation.clicked.connect(lambda: self.changeWidget('Segmentation'))
        self.btnSegmentation.setStyleSheet('QPushButton:checked,QPushButton:pressed {border: 1px solid #25101C; background-color: #BA5190; color: #FFFFFF}')
        mainLayoutRight.addWidget(self.btnSegmentation)

        # --others--
        self.btnCompare = QPushButton("Compare", self)
        self.btnCompare.clicked.connect(self.compareSegmentationWithFile)
        mainLayoutRight.addWidget(self.btnCompare)

        # --others--
        keyword = "3D Visualization"
        tmp = QPushButton(keyword, self)
        # tmp.clicked.connect(self.action3DVisualizationWidget)
        tmp.clicked.connect(lambda: self.changeWidget(keyword))
        mainLayoutRight.addWidget(tmp)
        self.ui_buttons[keyword] = tmp

        import imtools.show_segmentation_qt
        self.ui_widgets[keyword] = imtools.show_segmentation_qt.ShowSegmentationWidget(None)
        # import imtools.show_segmentation_qt as itss
        # itss.ShowSegmentationWidget()

        if self.oseg.debug_mode:
            btn_debug = QPushButton("Debug", self)
            btn_debug.clicked.connect(self.run_debug)
            mainLayoutRight.addWidget(btn_debug)

        btnQuit = QPushButton("Quit", self)
        btnQuit.clicked.connect(self.quit)
        mainLayoutRight.addWidget(btnQuit)

        mainLayoutRight.addStretch()

    def unlockAllButtons(self):
        self.btnSave.setDisabled(False)
        self.btnSegmentation.setDisabled(False)
        self.btnCompare.setDisabled(False)

    def changeWidget(self, option):
        # widgets = [
        #     self.infoBody,
        #     self.segBody,
        #     self.slabBody,
        #     self.read_widget
        # ]
        for key, widget in self.ui_widgets.items():

            widget.hide()

        if option == 'EditSlab':
            # self.infoBody.hide()
            # self.segBody.hide()
            self.slabBody.show()
        elif option == 'Main':
            self.infoBody.show()
            # self.segBody.hide()
            # self.slabBody.hide()
        elif option == 'Segmentation':
            if self.btnSegmentation.isChecked() == True:
                # self.infoBody.hide()
                self.segBody.reinitLabels()
                self.segBody.show()
                # self.slabBody.hide()
            else:
                self.infoBody.show()
            return
        elif option == "3D Visualization":
            # remove old
            import imtools.show_segmentation_qt
            widget = self.ui_widgets[option]
            self.bodyLayout.removeWidget(widget)
            widget.deleteLater()
            widget = None

            # add new
            vtk_file = self.oseg.get_standard_ouptut_filename(suffix="_{}", filetype="vtk")
            widget = imtools.show_segmentation_qt.ShowSegmentationWidget(None, show_load_interface=True, vtk_file=vtk_file )
            self.ui_widgets[option] = widget
            self.bodyLayout.addWidget(widget)
            widget.add_data(self.oseg.segmentation, self.oseg.voxelsize_mm, self.oseg.slab)
            widget.show()

        elif option == 'Load':
            self.read_widget.show()
            # self.infoBody.hide()
            # self.segBody.show()
            # self.slabBody.hide()

        elif option in self.ui_widgets.keys():
            self.ui_widgets[option].show()

        self.btnSegmentation.setChecked(False)

    def quit(self, event):
        return self.close()

    def run_debug(self, event):
        """
        Stop processing and start debugger in console. Function started by
        pressing "Debug" button wich is availeble after starting Lisa with
        -d parameter.
        """
        logger.debug('== Starting debug mode, leave it with command "c" =')
        from PyQt4.QtCore import pyqtRemoveInputHook
        pyqtRemoveInputHook()
        import ipdb; ipdb.set_trace()  # noqa BREAKPOINT

    def changeVoxelSize(self, val):
        self.scaling_mode = str(val)

    def setLabelText(self, obj, text):
        dlab = str(obj.text())
        obj.setText(dlab[:dlab.find(':')] + ': %s' % text)

    def getDcmInfo(self):
        vx_size = self.oseg.voxelsize_mm
        vsize = tuple([float(ii) for ii in vx_size])
        ret = ' %dx%dx%d,  %fx%fx%f mm' % (self.oseg.data3d.shape + vsize)

        return ret

    # def setVoxelVolume(self, vxs):
    #     self.voxel_volume = np.prod(vxs)

    def __get_datafile(self, app=False, directory=''):
        """
        Draw a dialog for directory selection.
        """

        from PyQt4.QtGui import QFileDialog
        if app:
            dcmdir = QFileDialog.getOpenFileName(
                caption='Select Data File',
                directory=directory
                # ptions=QFileDialog.ShowDirsOnly,
            )
        else:
            app = QApplication(sys.argv)
            dcmdir = QFileDialog.getOpenFileName(
                caption='Select DICOM Folder',
                # ptions=QFileDialog.ShowDirsOnly,
                directory=directory
            )
            # pp.exec_()
            app.exit(0)
        if len(dcmdir) > 0:

            dcmdir = "%s" % (dcmdir)
            dcmdir = dcmdir.encode("utf8")
        else:
            dcmdir = None
        return dcmdir

    def __get_datadir(self, app=False, directory=''):
        """
        Draw a dialog for directory selection.
        """

        from PyQt4.QtGui import QFileDialog
        if app:
            dcmdir = QFileDialog.getExistingDirectory(
                caption='Select DICOM Folder',
                options=QFileDialog.ShowDirsOnly,
                directory=directory
            )
        else:
            app = QApplication(sys.argv)
            dcmdir = QFileDialog.getExistingDirectory(
                caption='Select DICOM Folder',
                options=QFileDialog.ShowDirsOnly,
                directory=directory
            )
            # pp.exec_()
            app.exit(0)
        if len(dcmdir) > 0:

            dcmdir = "%s" % (dcmdir)
            dcmdir = dcmdir.encode("utf8")
        else:
            dcmdir = None
        return dcmdir

    def loadDataFile(self):
        self.statusBar().showMessage('Reading data file...')
        QApplication.processEvents()

        oseg = self.oseg
        if 'loadfiledir' in self.oseg.cache.data.keys():
            directory = self.oseg.cache.get('loadfiledir')
        else:
            directory = self.oseg.input_datapath_start
        #
        oseg.datapath = self.__get_datafile(
            app=True,
            directory=directory
        )

        if oseg.datapath is None:
            self.statusBar().showMessage('No data path specified!')
            return
        head, teil = os.path.split(oseg.datapath)
        self.oseg.cache.update('loadfiledir', head)

        self.importDataWithGui()

    def _before_read_callback(self, readerWidget):
        self.statusBar().showMessage('Reading data file...')
        QApplication.processEvents()

    def _after_read_callback(self, readerWidget):
        logger.debug("after read callback started")
        self.statusBar().showMessage('Ready')
        QApplication.processEvents()
        self.oseg.datapath = readerWidget.datapath
        self.importDataWithGui(datap=readerWidget.datap)
        logger.debug("after read callback finished")
        # print readerWidget.loaddir
        # print readerWidget.loadfiledir
        # import ipdb; ipdb.set_trace()

    def loadDataDir(self):
        self.statusBar().showMessage('Reading DICOM directory...')
        QApplication.processEvents()

        oseg = self.oseg
        if 'loaddir' in self.oseg.cache.data.keys():
            directory = self.oseg.cache.get('loaddir')
        else:
            directory = self.oseg.input_datapath_start

        oseg.datapath = self.__get_datadir(
            app=True,
            directory=directory
        )

        if oseg.datapath is None:
            self.statusBar().showMessage('No DICOM directory specified!')
            return
        # head, teil = os.path.split(oseg.datapath)
        self.oseg.cache.update('loaddir', oseg.datapath)

        self.importDataWithGui()

    def importDataWithGui(self, datap=None):
        logger.debug("import data with gui started")
        oseg = self.oseg

        if datap is None:
            reader = datareader.DataReader()

            # from PyQt4.QtCore import pyqtRemoveInputHook
            # pyqtRemoveInputHook()
            # seg.data3d, metadata =
            logger.debug("reading data")
            datap = reader.Get3DData(oseg.datapath, dataplus_format=True, gui=True, qt_app=self.qapp)
            logger.debug("datap readed")

        logger.debug("importing datap")
        oseg.import_dataplus(datap)
        self.statusBar().showMessage('Ready. Data loaded from ' + str(oseg.datapath))
        logger.debug("import data with gui finished")

        #### SET BUTTONS/MENU ####
        self.unlockAllButtons()


    def sync_lisa_data(self):
        """
        set sftp_username and sftp_password in ~/lisa_data/organ_segmentation.config
        """
        self.statusBar().showMessage('Sync in progress...')
        login = loginWindow.Login(checkLoginFcn=self.__loginCheckFcn)
        login.exec_()
        # print repr(self.oseg.sftp_username)
        # print repr(self.oseg.sftp_password)
        try:
            self.oseg.sync_lisa_data(self.oseg.sftp_username, self.oseg.sftp_password, callback=self._print_sync_progress)
        except:
            import traceback
            traceback.print_exc()
            logger.error(traceback.format_exc())

            QtGui.QMessageBox.warning(
                self, 'Error', 'Sync error')

        self.oseg.sftp_username = ''
        self.oseg.sftp_password = ''
        self.statusBar().showMessage('Sync finished')

    def __loginCheckFcn(self, textname, textpass):
        self.oseg.sftp_username = textname
        self.oseg.sftp_password = textpass
        return True

    def _print_sync_progress(self, transferred, toBeTransferred):
        self.statusBar().showMessage('Sync of current file {0} % '.format((100.0 * transferred) / toBeTransferred ))

        # print "Transferred: {0}\tOut of: {1}".format(transferred, toBeTransferred)

    def cropDcm(self):
        oseg = self.oseg

        if oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return

        self.statusBar().showMessage('Cropping DICOM data...')
        QApplication.processEvents()

        pyed = QTSeedEditor(oseg.data3d, mode='crop',
                            voxelSize=oseg.voxelsize_mm)
        # @TODO
        mx = self.oseg.viewermax
        mn = self.oseg.viewermin
        width = mx - mn
        # enter = (float(mx)-float(mn))
        center = np.average([mx, mn])
        logger.debug("window params max %f min %f width, %f center %f" %
                     (mx, mn, width, center))
        pyed.changeC(center)
        pyed.changeW(width)
        pyed.exec_()

        crinfo = pyed.getROI()
        if crinfo is not None:
            tmpcrinfo = []
            for ii in crinfo:
                tmpcrinfo.append([ii.start, ii.stop])

            # seg.data3d = qmisc.crop(oseg.data3d, oseg.crinfo)
            oseg.crop(tmpcrinfo)

        # self.setLabelText(self.text_dcm_data, self.getDcmInfo())
        self.statusBar().showMessage('Ready. Image cropped to ' + str(self.getDcmInfo()) )

    def maskRegion(self):
        if self.oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return

        self.statusBar().showMessage('Mask region...')
        QApplication.processEvents()

        pyed = QTSeedEditor(
                self.oseg.data3d, mode='mask',
                voxelSize=self.oseg.voxelsize_mm,
                contours=((self.oseg.segmentation == 0).astype(np.int8)*2)
                )

        pyed.contours_old = pyed.contours.copy()
        # initial mask set
        # pyed.masked = np.ones(self.oseg.data3d.shape, np.int8)
        # pyed.masked = (self.oseg.segmentation == 0).astype(np.int8)

        mx = self.oseg.viewermax
        mn = self.oseg.viewermin
        width = mx - mn
        # enter = (float(mx)-float(mn))
        center = np.average([mx, mn])
        logger.debug("window params max %f min %f width, %f center %f" %
                     (mx, mn, width, center))
        pyed.changeC(center)
        pyed.changeW(width)
        pyed.exec_()

        self.statusBar().showMessage('Ready')

    def action_add_segmentation_by_convex_areas(self):


        # @todo add button for this functionality
        nlabel, slabel = self.ui_select_label("Select label for segmentation")

        if self.oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return

        self.statusBar().showMessage('Draw segmentation by vonvex areas ...')

        QApplication.processEvents()

        target_segmentation = ((self.oseg.select_label(nlabel)).astype(np.int8)*2)
        # ed = sed3.sed3(target_segmentation)
        # ed.show()

        pyed = QTSeedEditor(
            self.oseg.data3d, mode='mask',
            voxelSize=self.oseg.voxelsize_mm,
            contours=target_segmentation
        )

        pyed.contours_old = pyed.contours.copy()
        # initial mask set
        # pyed.masked = np.ones(self.oseg.data3d.shape, np.int8)
        # pyed.masked = (self.oseg.segmentation == 0).astype(np.int8)

        mx = self.oseg.viewermax
        mn = self.oseg.viewermin
        width = mx - mn
        # enter = (float(mx)-float(mn))
        center = np.average([mx, mn])
        logger.debug("window params max %f min %f width, %f center %f" %
                     (mx, mn, width, center))
        pyed.changeC(center)
        pyed.changeW(width)
        pyed.exec_()

        contours = pyed.contours.copy()
        logger.debug("output contours unique" +
                     str(np.unique(contours)))
        self.segmentation_temp = contours
        # nlabel = self.oseg.nlabels(nlabel)
        self.oseg.segmentation_replacement(
            contours,
            label=nlabel,
            label_new=2,

        )
        # self.oseg.segmentation[contours == 2] = nlabel

        self.statusBar().showMessage('Ready')

    def _ui_run_with(self, fcn, init_msg=None, finish_msg=None):
        if init_msg is not None:
            self.statusBar().showMessage(init_msg)
        fcn()
        if finish_msg is not None:
            self.statusBar().showMessage(finish_msg)

    def action_segmentation_relabel(self):
        self.statusBar().showMessage('Relabelling segmentation')
        no, from_label = self.ui_select_label("Select from_label for renaming", multiple_choice=True)
        no, to_label = self.ui_select_label("Select to_label for renaming")
        self.oseg.segmentation_relabel(from_label=from_label, to_label=to_label)
        self.statusBar().showMessage('Ready. Relabeled from ' + str(from_label) + " to " + str(to_label))

    def mask_segmentation(self):
        # @todo add button for this functionality
        if self.oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return

        self.statusBar().showMessage('Mask region...')
        QApplication.processEvents()

        pyed = QTSeedEditor(
            self.oseg.segmentation, mode='mask',
            voxelSize=self.oseg.voxelsize_mm,
            contours=((self.oseg.segmentation == 0).astype(np.int8)*2)
        )

        pyed.contours_old = pyed.contours.copy()
        # initial mask set
        # pyed.masked = np.ones(self.oseg.data3d.shape, np.int8)
        # pyed.masked = (self.oseg.segmentation == 0).astype(np.int8)

        mx = self.oseg.viewermax
        mn = self.oseg.viewermin
        width = mx - mn
        # enter = (float(mx)-float(mn))
        center = np.average([mx, mn])
        logger.debug("window params max %f min %f width, %f center %f" %
                     (mx, mn, width, center))
        pyed.changeC(center)
        pyed.changeW(width)
        pyed.exec_()

        self.statusBar().showMessage('Ready')

    def btnLoadSegmentationFromFile(self):
        """
        Function make GUI for reading segmentaion file and calls
        organ_segmentation function to do the work.
        """
        self.statusBar().showMessage('Reading segmentation from file ...')
        QApplication.processEvents()
        logger.debug("import segmentation from file")
        logger.debug(str(self.oseg.crinfo))
        logger.debug(str(self.oseg.data3d.shape))
        logger.debug(str(self.oseg.segmentation.shape))
        path = self.oseg.cache.get_or_none('loadfiledir')
        if path is None:
            path = self.oseg.input_datapath_start
        seg_path = self.__get_datafile(
            app=True,
            directory=path
        )
        if seg_path is None:
            self.statusBar().showMessage('No data path specified!')
            return
        self.oseg.import_segmentation_from_file(seg_path)
        self.segBody.enableSegType()
        self.statusBar().showMessage('Ready. Segmentation loaded from ' + str(seg_path))

    def action_split_on_bifurcation(self):

        self.statusBar().showMessage('Split vessel tree...')
        ed = QTSeedEditor(img=self.oseg.data3d, contours=self.oseg.segmentation,
                          voxelSize=self.oseg.voxelsize_mm, init_brush_index=0)
        ed.exec_()
        seeds = ed.getSeeds()
        # ed.see
        # ed = sed3.sed3(self.oseg.segmentation)
        # ed.show()

        seeds = ed.seeds
        labeled_branches = self.oseg.segmentation

        # trunk_label = labeled_branches[seeds == 1][0]
        # branch_label1 = labeled_branches[seeds == 2][0]
        # branch_label2 = labeled_branches[seeds == 3][0]
        organ_label, textl = self.ui_select_label("Organ to split")

        # split_label, textl = self.ui_select_label("Label for new split")

        un = np.unique(seeds)
        import imma.labeled
        unlab = imma.labeled.unique_labels_by_seeds(self.oseg.segmentation, seeds)

        self.oseg.split_tissue_with_labeled_volumetric_vessel_tree(organ_label, trunk_label=unlab[1][0], branch_labels=unlab[2])  # trunk_label, branch_label1, branch_label2)
        self.statusBar().showMessage('Ready.')

    def __evaluation_to_text(self, evaluation):
        overall_score = evaluation['sliver_overall_pts']
        # \
        #     volumetry_evaluation.sliver_overall_score_for_one_couple(
        #         score
        #     )

        logger.info('overall score: ' + str(overall_score))
        return "Sliver score: " + str(overall_score)

    def ui_select_label(self, headline, text_inside="select from existing labels or write a new one",
                        return_i=True, return_str=True, multiple_choice=False):
        """ Get label with GUI.

        :return: numeric_label, string_label
        """

        # import copy
        # texts = copy.copy(self.oseg.slab.keys())

        strlab = self.ui_select_from_list(
            list(self.oseg.slab.keys()),
            headline,
            text_inside=text_inside,
            multiple_choice=multiple_choice
        )
        numlab = self.oseg.nlabels(strlab)
        if return_i and return_str:
            return numlab, strlab
        elif return_str:
            return strlab
        else:
            return numlab

    def ui_select_from_list(self,some_list, headline, text_inside="", multiple_choice=False):
        """ Select string from list with GUI.

        :return: selected string
        """

        if multiple_choice:
            slab_dialog = QtGui.QDialog(self)
            layout = QtGui.QGridLayout()
            slab_dialog.setLayout(layout)
            slab_wg = SelectLabelWidget(show_ok_button=False)
            slab_wg.init_slab(slab=self.oseg.slab, show_ok_button=False)
            slab_wg.update_slab_ui()

            layout.addWidget(slab_wg)
            ok = QPushButton("ok")
            ok.clicked.connect(slab_dialog.close)
            layout.addWidget(ok)
            self._ok_button_to_click_for_testing = ok
            slab_dialog.exec_()

            labels = slab_wg.get_selected_labels()
            labels = self.oseg.nlabels(labels, return_mode="str")
            strlab = labels
        else:
            strlab, ok = \
                QInputDialog.getItem(self,
                                     # self.qapp,
                                     headline,
                                     text_inside,
                                     some_list,
                                     editable=True)

            if not ok:
                raise ValueError("Selection canceled")
            strlab = str(strlab)

        # print("strlab", strlab)
        return strlab

    def ui_get_double(self, headline, value=0.0, **kwargs):
        """

        :param headline:
        :return: double_value
        """
        val, ok = \
            QInputDialog.getDouble(
                self,
                # self.qapp,
                headline,
                "insert number",
                value=value,
                **kwargs
            )
        return val, ok

    def compareSegmentationWithFile(self):
        """
        Function make GUI for reading segmentaion file to compare it with
        actual segmentation using Sliver methodics. It calls
        organ_segmentation function to do the work.
        """
        self.statusBar().showMessage('Reading segmentation from file ...')
        QApplication.processEvents()
        logger.debug("import segmentation from file to compare by sliver")
        logger.debug(str(self.oseg.crinfo))
        logger.debug(str(self.oseg.data3d.shape))
        logger.debug(str(self.oseg.segmentation.shape))
        if 'loadcomparedir' in self.oseg.cache.data.keys():
            directory = self.oseg.cache.get('loadcomparedir')
        else:
            directory = self.oseg.input_datapath_start
        #
        seg_path = self.__get_datafile(
            app=True,
            directory=directory
        )
        if seg_path is None:
            self.statusBar().showMessage('No data path specified!')
            return
        evaluation, segdiff = \
            self.oseg.sliver_compare_with_other_volume_from_file(seg_path)
        print('Evaluation: ', evaluation)
        # print 'Score: ', score

        text = self.__evaluation_to_text(evaluation)

        segdiff[segdiff == -1] = 2
        logger.debug('segdif unique ' + str(np.unique(segdiff)))

        QApplication.processEvents()
        try:

            ed = sed3.sed3qt(
                self.oseg.data3d,
                seeds=segdiff,
                # contour=(self.oseg.segmentation == self.oseg.slab['liver'])
                contour=self.oseg.segmentation
            )
            ed.exec_()
        except:
            ed = sed3.sed3(
                self.oseg.data3d,
                seeds=segdiff,
                contour=(self.oseg.segmentation == self.oseg.slab['liver'])
            )
            ed.show()

        head, teil = os.path.split(seg_path)
        self.oseg.cache.update('loadcompatext_seg_dataredir', head)
        self.setLabelText(self.segBody.lblSegData, text)
        self.statusBar().showMessage('Ready')

    def action_resize_mm(self):
        self.statusBar().showMessage('Performing resize ...')
        val, ok = self.ui_get_double(headline="Set new voxelsize [mm]", value=5.0)
        self.oseg.resize_to_mm(val)

        self.statusBar().showMessage('Ready')

    def btnSemiautoSeg(self):
        self.statusBar().showMessage('Performing liver segmentation ...')
        if self.oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return

        self.oseg.interactivity(
            min_val=self.oseg.viewermin,
            max_val=self.oseg.viewermax),
            # TODO
            # self.sublayout_segmentace
        self.checkSegData('auto. seg., ')
        self.statusBar().showMessage('Ready')

    def btnAutoSeg(self):
        self.statusBar().showMessage('Performing automatic segmentation...')
        QApplication.processEvents()
        if self.oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return

        self.oseg.run_sss()
        self.statusBar().showMessage('Automatic segmentation finished')

    def btnManualSeg(self):
        oseg = self.oseg
        # rint 'ms d3d ', oseg.data3d.shape
        # rint 'ms seg ', oseg.segmentation.shape
        # rint 'crinfo ', oseg.crinfo
        if oseg.data3d is None:
            self.statusBar().showMessage('No DICOM data!')
            return
        sgm = oseg.segmentation.astype(np.uint8)

        pyed = QTSeedEditor(oseg.data3d,
                            seeds=sgm,
                            mode='draw',
                            voxelSize=oseg.voxelsize_mm, volume_unit='ml')
        pyed.exec_()

        oseg.segmentation = pyed.getSeeds()
        self.oseg.processing_time = \
            datetime.datetime.now() - self.oseg.time_start
        self.checkSegData('manual seg., ')

    def checkSegData(self, msg):
        oseg = self.oseg
        if oseg.segmentation is None:
            self.statusBar().showMessage('No segmentation!')
            return

        nzs = oseg.segmentation.nonzero()
        nn = nzs[0].shape[0]
        if nn > 0:
            voxelvolume_mm3 = np.prod(oseg.voxelsize_mm)
            tim = self.oseg.processing_time

            if self.oseg.volume_unit == 'ml':
                # import datetime
                # timstr = str(datetime.timedelta(seconds=round(tim)))
                timstr = str(tim)
                # logger.debug('tim = ' + str(tim))
                aux = 'volume = %.2f [ml] , time = %s' %\
                      (nn * voxelvolume_mm3 / 1000, timstr)
            else:
                aux = 'volume = %.6e mm3' % (nn * voxelvolume_mm3, )
            self.setLabelText(self.segBody.lblSegData, msg + aux)
            self.statusBar().showMessage('Ready')

        else:
            self.statusBar().showMessage('No segmentation!')

    def saveOutImageStack(self, event=None):
        """
        Open dialog with filename suggestion to save output as image stack
        :param event:
        :return:
        """
        self.saveOut(image_stack=True)

    def ui_select_output_filename(self, filetype="pklz", suffix="_seeds", window_title="Save file", filter=None):
        if filter is None:
            if filetype is None:
                filter = "Former Lisa Format (*.pklz);;New Lisa format HDF5 (*.h5 *.hdf5);; Dicom (*.dcm);; All files(*.*)"
            elif filetype in ("pklz"):
                filter = "Former Lisa Format (*.pklz);; All files (*.*)"
            elif filetype in ("h5", "hdf5"):
                filter = "New Lisa format HDF5 (*.h5 *.hdf5);; All files (*.*)"
            elif filetype in ("dcm", "dicom"):
                filter = "Dicom (*.dcm);; All files (*.*)"
            else:
                filter = "(*.{});; All files (*.*)".format(filetype)
        ofilename = self.oseg.get_standard_ouptut_filename(filetype=filetype, suffix=suffix)
        filename = str(QFileDialog.getSaveFileName(
            self,
            window_title,
            ofilename,
            filter=filter))

        logger.info('Selected file: %s', filename)
        return filename

    def saveOut(self, event=None, filename=None, image_stack=False):
        """
        Open dialog for selecting file output filename. Uniqe name is as
        suggested.
        """
        if self.oseg.segmentation is not None:
            self.statusBar().showMessage('Saving segmentation data...')
            QApplication.processEvents()
            if image_stack:
                suffix = "{:04d}"
                ofilename = self.oseg.get_standard_ouptut_filename(filetype="dcm", suffix=suffix)
            else:
                ofilename = self.oseg.get_standard_ouptut_filename()
            filename = str(QFileDialog.getSaveFileName(
                self,
                "Save file",
                ofilename,
                filter="Former Lisa Format (*.pklz);;New Lisa format HDF5 (*.h5 *.hdf5);; Dicom (*.dcm)"))

            logger.info('Data saved to: ' + filename)

            self.oseg.save_outputs(filename)
            self.statusBar().showMessage('Ready. Data saved to ' + str(filename))

        else:
            self.statusBar().showMessage('No segmentation data!')

    def btnLoadSegmentationFromDicomOverlay(self, event=None):

        self.statusBar().showMessage('Reading dicom overlay')
        dirpath = self.oseg.load_segmentation_from_dicom_overlay(None)
        self.statusBar().showMessage('Ready. Dicom overlay loaded from ' + str(dirpath))

    def btnUpdate(self, event=None):

        self.statusBar().showMessage('Checking for update ...')
        self.oseg.update()
        self.statusBar().showMessage('Update finished. Please restart Lisa')

    def btnAutomaticLiverSeeds(self, event=None):
        self.statusBar().showMessage('Automatic liver seeds...')
        self.oseg.automatic_liver_seeds()
        self.statusBar().showMessage('Ready')

    def btnSaveSegmentation(self, event=None):
        """
        Not fully implemented yet
        :param event:
        :return:
        """
        import dictGUI
        slab_selection = {}
        for label, value in  self.oseg.slab.items():
            slab_selection[label] = True

        slab_selection = dictGUI.dictGui(slab_selection)
        # TODO use some function from oseg to store

    def btnConfig(self, event=None):
        import config
        import organ_segmentation as los
        import lisaConfigGui as lcg
        d = los.lisa_config_init()

        newconf = lcg.configGui(d)

        if newconf is None:
# reset config
            os.remove(
                os.path.join(
                    d['output_datapath'],
                    'organ_segmentation.config')
            )
        else:
            config.save_config(
                newconf,
                os.path.join(
                    newconf['output_datapath'],
                    'organ_segmentation.config')
            )
        
        self.quit(event)

    def btnSaveOutDcmOverlay(self, event=None, filename=None):
        if self.oseg.segmentation is not None:
            self.statusBar().showMessage('Saving segmentation data...')
            QApplication.processEvents()

            output_dicom_dir = self.oseg.save_outputs_dcm_overlay()
            self.statusBar().showMessage('Ready. Data saved to ' + output_dicom_dir)

        else:
            self.statusBar().showMessage('No segmentation data!')

    def btnSaveJSON(self, event=None, filename=None):
        if self.oseg.segmentation is not None:
            self.statusBar().showMessage('Saving json data...')
            QApplication.processEvents()
            if filename==None:
                ofilename = self.oseg.get_standard_ouptut_filename(filetype='json')
                filename = str(QFileDialog.getSaveFileName(
                    self,
                    "Save file",
                    ofilename,
                    filter="*.*"))

            self.oseg.output_annotaion_file = filename
            self.oseg.json_annotation_export()
            self.statusBar().showMessage('Ready')
        else:
            self.statusBar().showMessage('No segmentation data!')

    def btnSaveOutDcm(self, event=None, filename=None):
        logger.info('Pressed button "Save Dicom"')
        self.statusBar().showMessage('Saving input data...')
        QApplication.processEvents()
        ofilename = self.oseg.get_standard_ouptut_filename(filetype='dcm')
        filename = str(QFileDialog.getSaveFileName(
            self,
            "Save file",
            ofilename,
            filter="*.*"))

        self.oseg.save_input_dcm(filename)
        logger.info('Input data saved to: ' + filename)
        self.statusBar().showMessage('Ready')
        if self.oseg.segmentation is not None:
            self.statusBar().showMessage('Saving segmentation data...')
            QApplication.processEvents()
            filename = filename[:-4] + '-seg' + filename[-4:]
            logger.debug('saving to file: ' + filename)
            # osfilename = self.oseg.get_standard_ouptut_filename(
            #     filetype='dcm')
            self.oseg.save_outputs_dcm(filename)

            self.statusBar().showMessage('Ready')
        else:
            self.statusBar().showMessage('No segmentation data!')

    def action_label_volumetric_vessel_tree(self):
        self.statusBar().showMessage('Performing branch label...')
        nlabel, slabel = self.ui_select_label("Select label with vessel")
        print("label", slabel)
        self.oseg.label_volumetric_vessel_tree(vessel_label=slabel)
        self.statusBar().showMessage('Ready. Vessel {} branches labeled. '.format(str(slabel)))

    def btnVirtualResectionPV(self):
        self._virtual_resection('PV', )

    def btnVirtualResectionPV_new(self):
        self._virtual_resection('PV_new',
                                label=self.oseg.get_slab_value("liver"),
                                vein=2
                                )

    def btnVirtualResectionPlanar(self):
        self._virtual_resection('planar')

    def _virtual_resection(self, method='planar', **kwargs):
        # mport vessel_cut

        self.statusBar().showMessage('Performing virtual resection ...')
        data = {'data3d': self.oseg.data3d,
                'segmentation': self.oseg.segmentation,
                'slab': self.oseg.slab,
                'voxelsize_mm': self.oseg.voxelsize_mm
                }
        cut = virtual_resection.resection(data, method=method)
        self.oseg.segmentation = cut['segmentation']
        self.oseg.slab = cut['slab']

        # rint

        voxelvolume_mm3 = np.prod(self.oseg.voxelsize_mm)
        v1 = np.sum(cut['segmentation'] == self.oseg.slab['liver'])
        v2 = np.sum(cut['segmentation'] == self.oseg.slab['resected_liver'])
        v1 = (v1) * voxelvolume_mm3
        v2 = (v2) * voxelvolume_mm3
        aux = "volume = %.4g l, %.4g/%.4g (%.3g/%.3g %% )" % (
            (v1 + v2) * 1e-6,
            (v1) * 1e-6,
            (v2) * 1e-6,
            100 * v1 / (v1 + v2),
            100 * v2 / (v1 + v2)
        )
        self.setLabelText(self.segBody.lblSegData, aux)


    def btnLesionLocalization(self):
        self.oseg.lesionsLocalization()

    def btnPortalVeinSegmentation(self):
        """
        Function calls segmentation.vesselSegmentation function.
        """

        self.statusBar().showMessage('Vessel segmentation ...')
        # self.oseg.nlabels(2, "porta")
        # self.oseg.nlabels(3, "hepatic_veins")
        self.oseg.nlabels("porta")
        self.oseg.nlabels("hepatic_veins")
        organ_numeric_label, string_label = self.ui_select_label("Organ label")
        vessel_numeric_label, string_label = self.ui_select_label("Vessel label")
        # from PyQt4.QtCore import pyqtRemoveInputHook
        # pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace()
        self.oseg.portalVeinSegmentation(inner_vessel_label=vessel_numeric_label, organ_label=organ_numeric_label)
        self.statusBar().showMessage('Ready')

    def __saveVesselTreeGui(self, textLabel):
        textLabel = self.oseg.nlabels(textLabel, return_mode="str")
        fn_yaml = self.oseg.get_standard_ouptut_filename(filetype='yaml', suffix='-vt-' + textLabel+".yaml")
        fn_vtk = self.oseg.get_standard_ouptut_filename(filetype='vtk', suffix='-vt-' + textLabel+".vtk")

        fn_yaml = str(QFileDialog.getSaveFileName(
            self,
            "Save YAML file ",
            fn_yaml,
            filter="*.yaml"))
        fn_vtk = str(QFileDialog.getSaveFileName(
            self,
            "Save VTK file",
            fn_vtk,
            filter="*.vtk"))
        self.oseg.saveVesselTree(textLabel, fn_yaml=fn_yaml, fn_vtk=fn_vtk)
        self.statusBar().showMessage('Ready')

    def btnSavePortalVeinTree(self):
        self.statusBar().showMessage('Saving vessel tree ...')
        QApplication.processEvents()
        textLabel = 'porta'
        self.__saveVesselTreeGui(textLabel)
        self.statusBar().showMessage('Ready')

    def btnSaveActualVesselTree(self):
        self.statusBar().showMessage('Saving vessel tree ...')
        QApplication.processEvents()
        textLabel = self.oseg.nlabels(self.oseg.output_label, return_mode="str")
        msg = 'Saving "' + textLabel +'" vessel tree ...'
        logger.debug(msg)
        self.statusBar().showMessage(msg)
        self.__saveVesselTreeGui(textLabel)
        self.statusBar().showMessage('Ready')

    def btnHepaticVeinsSegmentation(self):
        """
        Function calls segmentation.vesselSegmentation function.
        """
        self.statusBar().showMessage('Vessel segmentation ...')
        self.oseg.hepaticVeinsSegmentation()
        self.statusBar().showMessage('Ready')

    def btnEditSlab(self):
        # run gui
        self.changeWidget('EditSlab')
        # predtim
        print(self.oseg.slab)

        #from PyQt4.QtCore import pyqtRemoveInputHook
        #pyqtRemoveInputHook()
        #import ipdb; ipdb.set_trace()

        # potom
        print(self.oseg.slab)


    def action3DVisualizationWidget(self):
        self.changeWidget("3D Visualization")

    def btnLog(self):
        import logWindow
        import os.path as op
        fn = op.expanduser("~/lisa_data/lisa.log")
        form = logWindow.LogViewerForm(fn) #, qapp=self.app)
        form.show()
        form.exec_()

    def btnRandomRotate(self):
        self.oseg.random_rotate()

    def btnRotateZ(self):
        pass


    def onAlternativeSegmentationParams(self, text):
        self.oseg.update_parameters_based_on_label(str(text))

    def view3D(self):
        # rom seg2mesh import gen_mesh_from_voxels, mesh2vtk, smooth_mesh
        # rom viewer import QVTKViewer
        oseg = self.oseg
        if oseg.segmentation is not None:
            import show_segmentation

            show_segmentation.showSegmentation(
                oseg.segmentation==1,
                voxelsize_mm=oseg.voxelsize_mm,
                degrad=1,
                resize_mm=1.5
            )
            # pts, els, et = gen_mesh_from_voxels(oseg.segmentation,
            #                                     oseg.voxelsize_mm,
            #                                     etype='q', mtype='s')
            # pts = smooth_mesh(pts, els, et,
            #                   n_iter=10)
            # vtkdata = mesh2vtk(pts, els, et)
            # view = QVTKViewer(vtk_data=vtkdata)
            view.exec_()

        else:
            self.statusBar().showMessage('No segmentation data!')
